import struct

from spinnman.model.enums.cpu_state import CPUState
from spinnman.model.enums.run_time_error import RunTimeError
from spinnman.model.enums.mailbox_command import MailboxCommand

CPU_INFO_BYTES = 128
CPU_USER_0_START_ADDRESS = 112


class CPUInfo(object):
    """ Represents information about the state of a CPU
    """

    def __init__(self, x, y, p, cpu_data, offset):
        """
        :param x: The x-coordinate of a chip
        :type x: int
        :param y: The y-coordinate of a chip
        :type y: int
        :param p: The id of a core on the chip
        :type p: int
        :param cpu_data: A bytestring received from SDRAM on the board
        :type cpu_data: str
        """
        self._x = x
        self._y = y
        self._p = p

        (registers,                                                  # 32s 0
         self._processor_state_register, self._stack_pointer,
         self._link_register,                                        # 3I  32
         run_time_error, self._physical_cpu_id,                      # 2B  44
         state, self._application_id,                                # 2B  46
         self._application_mailbox_data_address,
         self._monitor_mailbox_data_address,                         # 2I  48
         application_mailbox_command, monitor_mailbox_command,       # 2B  56
         self._software_error_count,                                 # H   58
         self._software_source_filename_address,
         self._software_source_line_number, self._time,              # 3I  60
         self._application_name,                                     # 16s 72
         self._iobuf_address, self._software_version,                # 2I  88
         # skipped                                                   # 16x 96
         user0, user1, user2, user3                                  # 4I  112
         ) = struct.unpack_from(
             "< 32s 3I 2B 2B 2I 2B H 3I 16s 2I 16x 4I", cpu_data, offset)

        index = self._application_name.find('\0')
        if index != -1:
            self._application_name = self._application_name[0:index]

        self._registers = struct.unpack_from("<IIIIIIII", registers)
        self._run_time_error = RunTimeError(run_time_error)
        self._state = CPUState(state)

        self._application_mailbox_command = MailboxCommand(
            application_mailbox_command)
        self._monitor_mailbox_command = MailboxCommand(
            monitor_mailbox_command)
        self._user = [user0, user1, user2, user3]

    @property
    def x(self):
        """ The x-coordinate of the chip containing the core

        :return: The x-coordinate of the chip
        :rtype: int
        """
        return self._x

    @property
    def y(self):
        """ The y-coordinate of the chip containing the core

        :return: The y-coordinate of the chip
        :rtype: int
        """
        return self._y

    @property
    def p(self):
        """ The id of the core on the chip

        :return: The id of the core
        :rtype: int
        """
        return self._p

    @property
    def state(self):
        """ The current state of the core

        :return: The state of the core
        :rtype: :py:class:`spinnman.model.cpu_state.CPUState`
        """
        return self._state

    @property
    def physical_cpu_id(self):
        """ The physical id of this processor

        :return: The physical id of the processor
        :rtype: int
        """
        return self._physical_cpu_id

    @property
    def application_name(self):
        """ The name of the application running on the core

        :return: The name of the application
        :rtype: str
        """
        return self._application_name

    @property
    def application_id(self):
        """ The id of the application running on the core

        :return: The id of the application
        :rtype: int
        """
        return self._application_id

    @property
    def time(self):
        """ The time at which the application started

        :return: The time in seconds since 00:00:00 on the 1st January 1970
        :rtype: long
        """
        return self._time

    @property
    def run_time_error(self):
        """ The reason for a run time error

        :return: The run time error
        :rtype: :py:class:`spinnman.model.run_time_error.RunTimeError`
        """
        return self._run_time_error

    @property
    def application_mailbox_command(self):
        """ The command currently in the mailbox being sent from the monitor\
            processor to the application

        :return: The command
        :rtype: :py:class:`spinnman.model.mailbox_command.MailboxCommand`
        """
        return self._application_mailbox_command

    @property
    def application_mailbox_data_address(self):
        """ The address of the data in SDRAM for the application mailbox

        :return: The address of the data
        :rtype: int
        """
        return self._application_mailbox_data_address

    @property
    def monitor_mailbox_command(self):
        """ The command currently in the mailbox being sent from the\
            application to the monitor processor

        :return: The command
        :rtype: :py:class:`spinnman.model.mailbox_command.MailboxCommand`
        """
        return self._monitor_mailbox_command

    @property
    def monitor_mailbox_data_address(self):
        """ The address of the data in SDRAM of the monitor mailbox

        :return: The address of the data
        :rtype: int
        """
        return self._monitor_mailbox_data_address

    @property
    def software_error_count(self):
        """ The number of software errors counted

        :return: The number of software errors
        :rtype: int
        """
        return self._software_error_count

    @property
    def software_source_filename_address(self):
        """ The address of the filename of the software source

        :return: The filename
        :rtype: str
        """
        return self._software_source_filename_address

    @property
    def software_source_line_number(self):
        """ The line number of the software source

        :return: The line number
        :rtype: int
        """
        return self._software_source_line_number

    @property
    def processor_state_register(self):
        """ The value in the processor state register (PSR)

        :return: The PSR value
        :rtype: int
        """
        return self._processor_state_register

    @property
    def stack_pointer(self):
        """ The current stack pointer value (SP)

        :return: The SP value
        :rtype: int
        """
        return self._stack_pointer

    @property
    def link_register(self):
        """ The current link register value (LR)

        :return: The LR value
        :rtype: int
        """
        return self._link_register

    @property
    def registers(self):
        """ The current register values (r0 - r7)

        :return: An array of 8 values, one for each register
        :rtype: array of int
        """
        return self._registers

    @property
    def user(self):
        """ The current user values (user0 - user3)

        :return: An array of 4 values, one for each user value
        :rtype: array of int
        """
        return self._user

    @property
    def iobuf_address(self):
        """ The address of the IOBUF buffer in SDRAM

        :return: The address
        :rtype: int
        """
        return self._iobuf_address

    @property
    def software_version(self):
        """ The software version

        :return: The software version
        :rtype: int
        """
        return self._software_version

    def __str__(self):
        return "{}:{}:{:02n} {:18} {:16s} {:3n}".format(
            self.x, self.y, self.p, self._state.name, self._application_name,
            self._application_id)



from spinnman.messages.scp.impl.scp_sdram_de_alloc_request import \
    SCPSDRAMDeAllocRequest
from spinnman.processes.abstract_multi_connection_process \
    import AbstractMultiConnectionProcess


class DeAllocSDRAMProcess(AbstractMultiConnectionProcess):

    def __init__(self, connection_selector):
        AbstractMultiConnectionProcess.__init__(self, connection_selector)
        self._no_blocks_freed = None

    def de_alloc_sdram(self, x, y, app_id, base_address=None):

        # deallocate space in the SDRAM
        if base_address is not None:
            self._send_request(
                SCPSDRAMDeAllocRequest(x, y, app_id, base_address))
        else:
            self._send_request(SCPSDRAMDeAllocRequest(x, y, app_id),
                               self.handle_sdram_alloc_response)
        self._finish()
        self.check_for_error()

    def handle_sdram_alloc_response(self, response):
        self._no_blocks_freed = response.number_of_blocks_freed

    @property
    def no_blocks_freed(self):
        return self._no_blocks_freed



from spinnman.messages.scp.abstract_messages.abstract_scp_request\
    import AbstractSCPRequest
from spinnman.messages.sdp.sdp_flag import SDPFlag
from spinnman.messages.sdp.sdp_header import SDPHeader
from spinnman.messages.scp.scp_request_header import SCPRequestHeader
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.impl.scp_chip_info_response \
    import SCPChipInfoResponse


class SCPChipInfoRequest(AbstractSCPRequest):
    """ An SCP request to read the chip information from a core
    """

    def __init__(self, x, y, with_size=False):
        """

        :param x: The x-coordinate of the chip to read from, between 0 and 255
        :type x: int
        :param y: The y-coordinate of the chip to read from, between 0 and 255
        :type y: int
        :param with_size: True if the size should be included in the response,\
                    False if not
        :type with_size: bool
        """
        # Bits 0-4 + bit 6 = all information except size
        argument_1 = 0x5F
        if with_size:

            # Bits 0-6 = all information including size
            argument_1 = 0x7F

        AbstractSCPRequest.__init__(
            self, SDPHeader(
                flags=SDPFlag.REPLY_EXPECTED, destination_port=0,
                destination_cpu=0, destination_chip_x=x,
                destination_chip_y=y),
            SCPRequestHeader(command=SCPCommand.CMD_INFO),
            argument_1=argument_1)

    def get_scp_response(self):
        """ See\
            :py:meth:`spinnman.messages.scp.abstract_scp_request.AbstractSCPRequest.get_scp_response`
        """
        return SCPChipInfoResponse()



import functools
from spinnman.messages.scp.impl.scp_read_link_request import SCPReadLinkRequest
from spinnman.messages.scp.impl.scp_read_memory_request \
    import SCPReadMemoryRequest
from spinnman.processes.abstract_multi_connection_process \
    import AbstractMultiConnectionProcess
from spinnman import constants


class ReadMemoryProcess(AbstractMultiConnectionProcess):
    """ A process for reading memory
    """

    def __init__(self, connection_selector):
        AbstractMultiConnectionProcess.__init__(self, connection_selector)
        self._view = None

    def handle_response(self, offset, response):
        self._view[offset:offset + response.length] = response.data[
            response.offset:response.offset + response.length]

    def read_memory(self, x, y, p, base_address, length):
        return self._read_memory(
            base_address, length,
            functools.partial(SCPReadMemoryRequest, x=x, y=y, cpu=p))

    def read_link_memory(self, x, y, p, link, base_address, length):
        return self._read_memory(
            base_address, length,
            functools.partial(SCPReadLinkRequest, x=x, y=y, cpu=p, link=link))

    def _read_memory(self, base_address, length, packet_class):
        data = bytearray(length)
        self._view = memoryview(data)
        n_bytes = length
        offset = 0
        while n_bytes > 0:

            bytes_to_get = n_bytes
            if bytes_to_get > constants.UDP_MESSAGE_MAX_SIZE:
                bytes_to_get = constants.UDP_MESSAGE_MAX_SIZE

            response_handler = functools.partial(self.handle_response, offset)
            self._send_request(
                packet_class(base_address=base_address + offset,
                             size=bytes_to_get),
                response_handler)

            n_bytes -= bytes_to_get
            offset += bytes_to_get

        self._finish()
        self.check_for_error()
        return data



from enum import Enum


class SCPDPRICommand(Enum):
    """ SCP Dropped Packet Reinjection Commands
    """
    SET_ROUTER_TIMEOUT = (0, "Set the WAIT1 timeout of the router")
    SET_ROUTER_EMERGENCY_TIMEOUT = (1, "Set the WAIT2 timeout of the router")
    SET_PACKET_TYPES = (2, "Set the packet types to reinject")
    GET_STATUS = (3, "Get the status of the reinjector")
    RESET_COUNTERS = (4, "Reset the statistics counters")
    EXIT = (5, "Exit the processs")

    def __new__(cls, value, doc=""):
        obj = object.__new__(cls)
        obj._value_ = value
        return obj

    def __init__(self, value, doc=""):
        self._value_ = value
        self.__doc__ = doc



from spinnman.connections.udp_packet_connections.udp_connection\
    import UDPConnection
from spinnman import constants


class UDPIpAddressesConnection(UDPConnection):
    """ A connection that detects any UDP packet that is transmitted by \
        spinnaker boards prior to boot
    """

    def __init__(self, local_host=None,
                 local_port=constants.UDP_BOOT_CONNECTION_DEFAULT_PORT):
        UDPConnection.__init__(self, local_host=local_host,
                               local_port=local_port)

    def supports_sends_message(self, message):
        return False

    def receive_ip_address(self, timeout=None):
        try:
            (_, (ip_address, port)) = self.receive_with_address(timeout)
            if port == 54321:
                return ip_address
            return None
        except Exception:
            return None

    def __repr__(self):
        return \
            "UDPIpAddressesConnection(local_host={}, local_port={})".format(
                self.local_ip_address, self.local_port)



from enum import Enum


class SCPAllocFreeType(Enum):
    """ The SCP Allocation and Free codes
    """

    ALLOC_SDRAM = (0, "Allocate SDRAM")
    FREE_SDRAM_BY_POINTER = (1, "Free SDRAM using a Pointer")
    FREE_SDRAM_BY_TAG = (2, "Free SDRAM using a Tag")
    ALLOC_ROUTING = (3, "Allocate Routing Entries")
    FREE_ROUTING_BY_POINTER = (4, "Free Routing Entries by Pointer")
    FREE_ROUTING_BY_APP_ID = (5, "Free Routing Entries by APP ID")

    def __new__(cls, value, doc=""):
        obj = object.__new__(cls)
        obj._value_ = value
        return obj

    def __init__(self, value, doc=""):
        self._value_ = value
        self.__doc__ = doc



from spinnman.messages.scp.enums.scp_signal import SCPSignal
from spinnman.messages.scp.abstract_messages.abstract_scp_request \
    import AbstractSCPRequest
from spinnman.messages.sdp.sdp_header import SDPHeader
from spinnman.messages.sdp.sdp_flag import SDPFlag
from spinnman.messages.scp.scp_request_header import SCPRequestHeader
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.impl.scp_check_ok_response import SCPCheckOKResponse

_APP_MASK = 0xFF


def _get_data(app_id, signal):
    data = (_APP_MASK << 8) | app_id
    data += signal.value << 16
    return data


class SCPAppStopRequest(AbstractSCPRequest):
    """ An SCP Request to stop an application
    """

    def __init__(self, app_id):
        """

        :param app_id: The id of the application, between 0 and 255
        :type app_id: int
        """
        super(SCPAppStopRequest, self).__init__(
            SDPHeader(
                flags=SDPFlag.REPLY_EXPECTED, destination_port=0,
                destination_cpu=0,
                destination_chip_x=self.DEFAULT_DEST_X_COORD,
                destination_chip_y=self.DEFAULT_DEST_Y_COORD),
            SCPRequestHeader(command=SCPCommand.CMD_NNP),
            argument_1=(0x3f << 16),
            argument_2=(5 << 28) | _get_data(app_id, SCPSignal.STOP),
            argument_3=(1 << 31) + (0x3f << 8))

    def get_scp_response(self):
        return SCPCheckOKResponse("Send Stop", "CMD_NNP")



from spinnman.messages.scp.abstract_messages.abstract_scp_request \
    import AbstractSCPRequest
from spinnman.messages.sdp.sdp_header import SDPHeader
from spinnman.messages.sdp.sdp_flag import SDPFlag
from spinnman.messages.scp.scp_request_header import SCPRequestHeader
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.impl.scp_check_ok_response import SCPCheckOKResponse

_NNP_FORWARD_RETRY = (1 << 31) | (0x3f << 8) | 0x18
_NNP_FLOOD_FILL_START = 6


class SCPFloodFillStartRequest(AbstractSCPRequest):
    """ A request to start a flood fill of data
    """

    def __init__(self, nearest_neighbour_id, n_blocks, x=None, y=None):
        """

        :param nearest_neighbour_id: The id of the packet, between 0 and 127
        :type nearest_neighbour_id: int
        :param n_blocks: The number of blocks of data that will be sent,\
                    between 0 and 255
        :type n_blocks: int
        :param x: The x-coordindate of the chip to load the data on to.  If\
                    not specified, the data will be loaded on to all chips
        :type x: int
        :param y: The y-coordinate of the chip to load the data on to.  If\
                    not specified, the data will be loaded on to all chips
        :type y: int
        """
        key = ((_NNP_FLOOD_FILL_START << 24) | (nearest_neighbour_id << 16) |
               (n_blocks << 8))
        data = 0xFFFF
        if x is not None and y is not None:
            m = ((y & 3) * 4) + (x & 3)
            data = (((x & 0xfc) << 24) + ((y & 0xfc) << 16) +
                    (3 << 16) + (1 << m))

        super(SCPFloodFillStartRequest, self).__init__(
            SDPHeader(
                flags=SDPFlag.REPLY_EXPECTED, destination_port=0,
                destination_cpu=0,
                destination_chip_x=self.DEFAULT_DEST_X_COORD,
                destination_chip_y=self.DEFAULT_DEST_Y_COORD),
            SCPRequestHeader(command=SCPCommand.CMD_NNP),
            argument_1=key, argument_2=data, argument_3=_NNP_FORWARD_RETRY)

    def get_scp_response(self):
        return SCPCheckOKResponse("Flood Fill", "CMD_NNP:NNP_FFS")



from spinnman.exceptions import SpinnmanInvalidPacketException, \
    SpinnmanInvalidParameterTypeException
from spinnman.messages.eieio.command_messages.eieio_command_message\
    import EIEIOCommandMessage
from spinnman.messages.eieio.command_messages.eieio_command_header\
    import EIEIOCommandHeader
from spinnman import constants
import struct


class SpinnakerRequestReadData(EIEIOCommandMessage):
    """ Message used in the context of the buffering output mechanism which is\
        sent from the SpiNNaker system to the host computer to signal that\
        some data is available to be read
    """
    def __init__(self, x, y, p, region_id, sequence_no, n_requests,
                 channel, start_address, space_to_be_read):
        if not isinstance(channel, list):
            channel = [channel]

        if not isinstance(region_id, list):
            region_id = [region_id]

        if not isinstance(start_address, list):
            start_address = [start_address]

        if not isinstance(space_to_be_read, list):
            space_to_be_read = [space_to_be_read]

        if len(start_address) != n_requests or \
                len(space_to_be_read) != n_requests or \
                len(channel) != n_requests or \
                len(region_id) != n_requests:
            raise SpinnmanInvalidPacketException(
                "SpinnakerRequestReadData",
                "The format for a SpinnakerRequestReadData packet is "
                "invalid: {0:d} request(s), {1:d} start address(es) "
                "defined, {2:d} space(s) to be read defined, {3:d} region(s) "
                "defined, {4:d} channel(s) defined".format(
                    n_requests, len(start_address), len(space_to_be_read),
                    len(region_id), len(channel)))
        EIEIOCommandMessage.__init__(
            self, EIEIOCommandHeader(
                constants.EIEIO_COMMAND_IDS.SPINNAKER_REQUEST_READ_DATA.value))
        self._header = _SpinnakerRequestReadDataHeader(
            x, y, p, n_requests, sequence_no)
        self._requests = _SpinnakerRequestReadDataRequest(
            channel, region_id, start_address, space_to_be_read)

    @property
    def x(self):
        return self._header.x

    @property
    def y(self):
        return self._header.y

    @property
    def p(self):
        return self._header.p

    @property
    def n_requests(self):
        return self._header.n_requests

    @property
    def sequence_no(self):
        return self._header.sequence_no

    def channel(self, request_id):
        return self._requests.channel(request_id)

    def region_id(self, request_id):
        return self._requests.region_id(request_id)

    def start_address(self, request_id):
        return self._requests.start_address(request_id)

    def space_to_be_read(self, request_id):
        return self._requests.space_to_be_read(request_id)

    @staticmethod
    def get_min_packet_length():
        return 16

    @staticmethod
    def from_bytestring(command_header, data, offset):
        (y, x, processor_and_requests, sequence_no) = \
            struct.unpack_from("<BBBB", data, offset)
        p = (processor_and_requests >> 3) & 0x1F
        n_requests = processor_and_requests & 0x7
        offset += 4
        channel = list()
        region_id = list()
        start_address = list()
        space_to_be_read = list()

        for i in xrange(n_requests):
            if i == 0:
                request_channel, request_region_id, request_start_address, \
                    request_space_to_be_read = struct.unpack_from(
                        "<BBII", data, offset)
                offset += 10
            else:
                request_channel, request_region_id, request_start_address, \
                    request_space_to_be_read = struct.unpack_from(
                        "<xxBBII", data, offset)
                offset += 12
            channel.append(request_channel)
            region_id.append(request_region_id)
            start_address.append(request_start_address)
            space_to_be_read.append(request_space_to_be_read)
        return SpinnakerRequestReadData(
            x, y, p, region_id, sequence_no, n_requests, channel,
            start_address, space_to_be_read)

    @property
    def bytestring(self):
        byte_string = super(SpinnakerRequestReadData, self).bytestring
        byte_string += struct.pack("<BB", self.x, self.y)
        n_requests = self.n_requests
        processor_and_request = (self.p << 3) | n_requests

        for i in xrange(n_requests):
            if i == 0:
                byte_string += struct.pack(
                    "<BBBBII", processor_and_request, self.sequence_no,
                    self.channel(i), self.region_id(i),
                    self.start_address(0), self.space_to_be_read(i))
            else:
                byte_string += struct.pack(
                    "<xxBBII", self.channel(i), self.region_id(i),
                    self.start_address(0), self.space_to_be_read(i))
        return byte_string


class _SpinnakerRequestReadDataHeader(object):
    """ Contains the position of the core in the machine (x, y, p), the number\
        of requests and a sequence number
    """
    def __init__(self, x, y, p, n_requests, sequence_no):
        self._x = x
        self._y = y
        self._p = p
        self._n_requests = n_requests
        self._sequence_no = sequence_no

    @property
    def x(self):
        return self._x

    @property
    def y(self):
        return self._y

    @property
    def p(self):
        return self._p

    @property
    def sequence_no(self):
        return self._sequence_no

    @property
    def n_requests(self):
        return self._n_requests


class _SpinnakerRequestReadDataRequest(object):
    """ Contains a set of requests which refer to the channels used
    """
    def __init__(self, channel, region_id, start_address, space_to_be_read):
        if not isinstance(channel, list):
            self._channel = [channel]
        else:
            self._channel = channel

        if not isinstance(region_id, list):
            self._region_id = [region_id]
        else:
            self._region_id = region_id

        if not isinstance(start_address, list):
            self._start_address = [start_address]
        else:
            self._start_address = start_address

        if not isinstance(space_to_be_read, list):
            self._space_to_be_read = [space_to_be_read]
        else:
            self._space_to_be_read = space_to_be_read

    def channel(self, request_id):
        if len(self._channel) > request_id:
            return self._channel[request_id]
        else:
            SpinnmanInvalidParameterTypeException(
                "request_id", "integer", "channel request needs to be"
                "comprised between 0 and {0:d}; current value: "
                "{1:d}".format(len(self._channel) - 1, request_id))

    def region_id(self, request_id):
        if len(self._region_id) > request_id:
            return self._region_id[request_id]
        else:
            SpinnmanInvalidParameterTypeException(
                "request_id", "integer", "region id request needs to be"
                "comprised between 0 and {0:d}; current value: "
                "{1:d}".format(len(self._region_id) - 1, request_id))

    def start_address(self, request_id):
        if len(self._start_address) > request_id:
            return self._start_address[request_id]
        else:
            SpinnmanInvalidParameterTypeException(
                "request_id", "integer", "start address request needs to be"
                "comprised between 0 and {0:d}; current value: "
                "{1:d}".format(len(self._start_address) - 1, request_id))

    def space_to_be_read(self, request_id):
        if len(self._space_to_be_read) > request_id:
            return self._space_to_be_read[request_id]
        else:
            SpinnmanInvalidParameterTypeException(
                "request_id", "integer", "space to be read request needs to be"
                "comprised between 0 and {0:d}; current value: "
                "{1:d}".format(len(self._space_to_be_read) - 1, request_id))



from spinnman.messages.eieio.eieio_type import EIEIOType
from spinnman.messages.eieio.data_messages.eieio_without_payload_data_message\
    import EIEIOWithoutPayloadDataMessage
from spinnman.messages.eieio.data_messages.eieio_data_header\
    import EIEIODataHeader
from spinnman.messages.eieio.data_messages.eieio_data_message\
    import EIEIODataMessage


class EIEIO16BitDataMessage(EIEIOWithoutPayloadDataMessage):
    """ An EIEIO packet containing 16 bit events
    """
    def __init__(self, count=0, data=None, offset=0):

        EIEIOWithoutPayloadDataMessage.__init__(
            self, EIEIODataHeader(EIEIOType.KEY_16_BIT, count=count),
            data, offset)

    @staticmethod
    def get_min_packet_length():
        return EIEIODataMessage.min_packet_length(EIEIOType.KEY_16_BIT)



from spinnman.exceptions import SpinnmanInvalidParameterException
from spinnman import constants


class RouterDiagnostics(object):
    """ Represents a set of diagnostic information available from a chip router
    """

    def __init__(self, control_register, error_status, register_values):
        """
        :param control_register: The value of the control register
        :type control_register: int
        :param error_status: The value of the error_status
        :type error_status: int
        :param register_values: The values of the 16 router registers
        :type register_values: iterable of int
        :raise spinnman.exceptions.SpinnmanInvalidParameterException: If the\
                    number of register values is not 16
        """
        if len(register_values) != 16:
            raise SpinnmanInvalidParameterException(
                "len(register_values)",
                str(len(constants.ROUTER_REGISTER_REGISTERS)),
                "There must be exactly 16 register values")

        self._mon = (control_register >> 8) & 0x1F
        self._wait_1 = (control_register >> 16) & 0xFF
        self._wait_2 = (control_register >> 8) & 0xFF

        self._error_status = error_status

        self._register_values = register_values

    @property
    def mon(self):
        """ The "mon" part of the control register

        :return: The mon bits
        :rtype: int
        """
        return self._mon

    @property
    def wait_1(self):
        """ The wait_1 part of the control register

        :return: The wait_1 bits
        :rtype: int
        """
        return self._wait_1

    @property
    def wait_2(self):
        """ The wait_2 part of the control register

        :return: The wait_2 bits
        :rtype: int
        """
        return self._wait_2

    @property
    def error_status(self):
        """ The error status

        :return: The error status
        :rtype: int
        """
        return self._error_status

    @property
    def n_local_multicast_packets(self):
        """ The number of multicast packets received from local cores

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.LOC_MC.value]

    @property
    def n_external_multicast_packets(self):
        """ The number of multicast packets received from external links

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.EXT_MC.value]

    @property
    def n_dropped_multicast_packets(self):
        """ The number of multicast packets received that were dropped

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.DUMP_MC.value]

    @property
    def n_local_peer_to_peer_packets(self):
        """ The number of peer-to-peer packets received from local cores

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.LOC_PP.value]

    @property
    def n_external_peer_to_peer_packets(self):
        """ The number of peer-to-peer packets received from external links

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.EXT_PP.value]

    @property
    def n_dropped_peer_to_peer_packets(self):
        """ The number of peer-to-peer packets received that were dropped

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.DUMP_PP.value]

    @property
    def n_local_nearest_neighbour_packets(self):
        """ The number of nearest-neighbour packets received from local cores

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.LOC_NN.value]

    @property
    def n_external_nearest_neighbour_packets(self):
        """ The number of nearest-neighbour packets received from external\
            links

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.EXT_NN.value]

    @property
    def n_dropped_nearest_neighbour_packets(self):
        """ The number of nearest-neighbour packets received that were dropped

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.DUMP_NN.value]

    @property
    def n_local_fixed_route_packets(self):
        """ The number of fixed-route packets received from local cores

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.LOC_FR.value]

    @property
    def n_external_fixed_route_packets(self):
        """ The number of fixed-route packets received from external links

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.EXT_FR.value]

    @property
    def n_dropped_fixed_route_packets(self):
        """ The number of fixed-route packets received that were dropped

        :return: The number of packets
        :rtype: int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.DUMP_FR.value]

    @property
    def user_0(self):
        """ the data gained from the user 0 router diag filter

        :return: the number of packets captured by this filter.
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.USER_0.value]

    @property
    def user_1(self):
        """ the data gained from the user 1 router diag filter

        :return: the number of packets captured by this filter.
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.USER_1.value]

    @property
    def user_2(self):
        """ the data gained from the user 2 router diag filter

        :return: the number of packets captured by this filter.
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.USER_2.value]

    @property
    def user_3(self):
        """ the data gained from the user 3 router diag filter

        :return: the number of packets captured by this filter.
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.USER_3.value]

    @property
    def user_registers(self):
        """ The values in the user control registers

        :return: An array of 4 values
        :rtype: array of int
        """
        return self._register_values[
            constants.ROUTER_REGISTER_REGISTERS.USER_0.value:
            constants.ROUTER_REGISTER_REGISTERS.USER_3.value + 1]

    @property
    def registers(self):
        """ The values in all of the registers.  Can be used to directly\
            access the registers if they have been programmed to give\
            different values

        :return: An array of 16 values
        :rtype: array of int
        """
        return self._register_values



from spinnman.messages.scp.abstract_messages.abstract_scp_request\
    import AbstractSCPRequest
from spinnman.messages.sdp.sdp_flag import SDPFlag
from spinnman.messages.sdp.sdp_header import SDPHeader
from spinnman.messages.scp.scp_request_header import SCPRequestHeader
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.impl.scp_version_response import SCPVersionResponse


class SCPVersionRequest(AbstractSCPRequest):
    """ An SCP request to read the version of software running on a core
    """

    def __init__(self, x, y, p):
        """

        :param x: The x-coordinate of the chip to read from, between 0 and 255
        :type x: int
        :param y: The y-coordinate of the chip to read from, between 0 and 255
        :type y: int
        :param p: The id of the processor to read the version from,\
                    between 0 and 31
        :type p: int
        :raise spinnman.exceptions.SpinnmanInvalidParameterException:
                    * If the chip coordinates are out of range
                    * If the processor is out of range
        """
        super(SCPVersionRequest, self).__init__(
            SDPHeader(
                flags=SDPFlag.REPLY_EXPECTED, destination_port=0,
                destination_cpu=p, destination_chip_x=x,
                destination_chip_y=y),
            SCPRequestHeader(command=SCPCommand.CMD_VER))

    def get_scp_response(self):
        """ See\
            :py:meth:`spinnman.messages.scp.abstract_scp_request.AbstractSCPRequest.get_scp_response`
        """
        return SCPVersionResponse()



import struct

from spinnman.messages.scp.abstract_messages.abstract_scp_response\
    import AbstractSCPResponse
from spinnman.messages.scp.enums.scp_result import SCPResult
from spinnman.exceptions import SpinnmanUnexpectedResponseCodeException


class SCPSDRAMDeAllocResponse(AbstractSCPResponse):
    """ An SCP response to a request to deallocate SDRAM
    """

    def __init__(self):
        """
        """
        AbstractSCPResponse.__init__(self)
        self._number_of_blocks_freed = None

    def read_data_bytestring(self, data, offset):
        """ See\
            :py:meth:`spinnman.messages.scp.abstract_scp_response.AbstractSCPResponse.read_data_bytestring`
        """
        result = self.scp_response_header.result
        if result != SCPResult.RC_OK:
            raise SpinnmanUnexpectedResponseCodeException(
                "SDRAM deallocation", "CMD_DEALLOC", result.name)
        self._number_of_blocks_freed = struct.unpack_from(
            "<I", data, offset)[0]

        # check that the base address is not null (0 in python case) as
        # this reflects a issue in command on spinnaker side
        if self._number_of_blocks_freed == 0:
            raise SpinnmanUnexpectedResponseCodeException(
                "SDRAM deallocation response base address", "CMD_DEALLOC",
                result.name)

    @property
    def number_of_blocks_freed(self):
        """ The number of allocated blocks that have been freed from the\
            app_id given

        :rtype: int
        """
        return self._number_of_blocks_freed



import functools
import struct
from collections import defaultdict
from collections import OrderedDict

from spinnman.model.io_buffer import IOBuffer
from spinnman.utilities import utility_functions
from spinnman.messages.scp.impl.scp_read_memory_request \
    import SCPReadMemoryRequest
from spinnman.processes.abstract_multi_connection_process \
    import AbstractMultiConnectionProcess
from spinnman import constants


class ReadIOBufProcess(AbstractMultiConnectionProcess):
    """ A process for reading memory
    """

    def __init__(self, connection_selector):
        AbstractMultiConnectionProcess.__init__(self, connection_selector)

        # A dictionary of (x, y, p) -> iobuf address
        self._iobuf_address = dict()

        # A dictionary of (x, y, p) -> OrderedDict(n) -> bytearray
        self._iobuf = defaultdict(OrderedDict)

        # A dictionary of (x, y, p) -> OrderedDict(n) -> memoryview
        self._iobuf_view = defaultdict(OrderedDict)

        # A list of extra reads that need to be done as a result of the first
        # read = list of (x, y, p, n, base_address, size, offset)
        self._extra_reads = list()

        # A list of next reads that need to be done as a result of the first
        # read = list of (x, y, p, n, next_address, first_read_size)
        self._next_reads = list()

    def handle_iobuf_address_response(self, iobuf_size, x, y, p, response):
        iobuf_address = struct.unpack_from(
            "<I", response.data, response.offset)[0]
        if iobuf_address != 0:
            first_read_size = min(
                (iobuf_size + 16, constants.UDP_MESSAGE_MAX_SIZE))
            self._next_reads.append((
                x, y, p, 0, iobuf_address, first_read_size))

    def handle_first_iobuf_response(self, x, y, p, n, base_address,
                                    first_read_size, response):

        # Unpack the iobuf header
        (next_address, bytes_to_read) = struct.unpack_from(
            "<I8xI", response.data, response.offset)

        # Create a buffer for the data
        data = bytearray(bytes_to_read)
        view = memoryview(data)
        self._iobuf[(x, y, p)][n] = data
        self._iobuf_view[(x, y, p)][n] = view

        # Put the data from this packet into the buffer
        packet_bytes = response.length - 16
        if packet_bytes > bytes_to_read:
            packet_bytes = bytes_to_read
        offset = response.offset + 16
        if packet_bytes > 0:
            view[0:packet_bytes] = response.data[
                offset:(offset + packet_bytes)]

        bytes_to_read -= packet_bytes
        base_address += packet_bytes + 16
        read_offset = packet_bytes

        # While more reads need to be done to read the data
        while bytes_to_read > 0:

            # Read the next bit of memory making up the buffer
            next_bytes_to_read = min((bytes_to_read,
                                      constants.UDP_MESSAGE_MAX_SIZE))
            self._extra_reads.append((x, y, p, n, base_address,
                                      next_bytes_to_read, read_offset))
            base_address += next_bytes_to_read
            read_offset += next_bytes_to_read
            bytes_to_read -= next_bytes_to_read

        # If there is another IOBuf buffer, read this next
        if next_address != 0:
            self._next_reads.append((x, y, p, n + 1, next_address,
                                     first_read_size))

    def handle_extra_iobuf_response(self, x, y, p, n, offset, response):
        view = self._iobuf_view[(x, y, p)][n]
        view[offset:offset + response.length] = response.data[
            response.offset:response.offset + response.length]

    def read_iobuf(self, iobuf_size, core_subsets):

        # Get the iobuf address for each core
        for core_subset in core_subsets:
            x = core_subset.x
            y = core_subset.y
            for p in core_subset.processor_ids:
                base_address = (
                    utility_functions.get_vcpu_address(p) +
                    constants.CPU_IOBUF_ADDRESS_OFFSET)
                self._send_request(
                    SCPReadMemoryRequest(x, y, base_address, 4),
                    functools.partial(
                        self.handle_iobuf_address_response,
                        iobuf_size, x, y, p))
        self._finish()
        self.check_for_error()

        # Run rounds of the process until reading is complete
        while len(self._extra_reads) > 0 or len(self._next_reads) > 0:

            # Process the extra iobuf reads needed
            while len(self._extra_reads) > 0:
                (x, y, p, n, base_address, size, offset) = \
                    self._extra_reads.pop()

                self._send_request(
                    SCPReadMemoryRequest(x, y, base_address, size),
                    functools.partial(self.handle_extra_iobuf_response,
                                      x, y, p, n, offset))

            # Process the next iobuf reads needed
            while len(self._next_reads) > 0:
                (x, y, p, n, next_address, first_read_size) = \
                    self._next_reads.pop()
                self._send_request(
                    SCPReadMemoryRequest(x, y, next_address, first_read_size),
                    functools.partial(self.handle_first_iobuf_response,
                                      x, y, p, n, next_address,
                                      first_read_size))

            # Finish this round
            self._finish()

        self.check_for_error()
        for core_subset in core_subsets:
            x = core_subset.x
            y = core_subset.y
            for p in core_subset.processor_ids:
                iobufs = self._iobuf[(x, y, p)]
                iobuf = ""
                for item in iobufs.itervalues():
                    iobuf += item.decode("ascii")

                yield IOBuffer(x, y, p, iobuf)



from spinnman import constants
from spinnman import exceptions
import logging

from spinnman.messages.scp.impl.scp_read_memory_request\
    import SCPReadMemoryRequest
from spinnman.messages.scp.impl.scp_read_link_request import SCPReadLinkRequest
from spinnman.model.p2p_table import P2PTable
from spinnman.messages.scp.impl.scp_chip_info_request import SCPChipInfoRequest
from spinnman.model.enums.cpu_state import CPUState
from spinnman.processes.abstract_multi_connection_process \
    import AbstractMultiConnectionProcess
from spinnman.processes.abstract_process import AbstractProcess
from spinn_machine.processor import Processor
from spinn_machine.router import Router
from spinn_machine.chip import Chip
from spinn_machine.sdram import SDRAM
from spinn_machine.machine import Machine
from spinn_machine.link import Link

logger = logging.getLogger(__name__)


class GetMachineProcess(AbstractMultiConnectionProcess):
    """ A process for getting the machine details over a set of connections
    """

    def __init__(self, connection_selector, ignore_chips, ignore_cores,
                 ignore_links, max_core_id, max_sdram_size=None):
        AbstractMultiConnectionProcess.__init__(self, connection_selector)

        self._ignore_chips = ignore_chips if ignore_chips is not None else {}
        self._ignore_cores = ignore_cores if ignore_cores is not None else {}
        self._ignore_links = ignore_links if ignore_links is not None else {}
        self._max_core_id = max_core_id
        self._max_sdram_size = max_sdram_size

        self._p2p_column_data = list()

        # A dictionary of (x, y) -> ChipInfo
        self._chip_info = dict()

    def _make_chip(self, width, height, chip_info):
        """ Creates a chip from a ChipSummaryInfo structure

        :param chip_info: The ChipSummaryInfo structure to create the chip\
                    from
        :type chip_info: \
                    :py:class:`spinnman.model.chip_info.ChipSummaryInfo`
        :return: The created chip
        :rtype: :py:class:`spinn_machine.chip.Chip`
        """

        # Create the processor list
        processors = list()
        max_core_id = chip_info.n_cores - 1
        core_states = chip_info.core_states
        if self._max_core_id is not None and max_core_id > self._max_core_id:
            max_core_id = self._max_core_id
        for virtual_core_id in range(max_core_id + 1):

            # Add the core provided it is not to be ignored
            if ((chip_info.x, chip_info.y, virtual_core_id) not in
                    self._ignore_cores):
                if virtual_core_id == 0:
                    processors.append(Processor(
                        virtual_core_id, is_monitor=True))
                elif core_states[virtual_core_id] == CPUState.IDLE:
                    processors.append(Processor(virtual_core_id))
                else:
                    logger.warn("Not using core {}, {}, {} in state {}".format(
                        chip_info.x, chip_info.y, virtual_core_id,
                        core_states[virtual_core_id]))

        # Create the router
        links = list()
        for link in chip_info.working_links:
            dest_x, dest_y = Machine.get_chip_over_link(
                chip_info.x, chip_info.y, link, width, height)
            if ((chip_info.x, chip_info.y, link) not in self._ignore_links and
                    (dest_x, dest_y) not in self._ignore_chips and
                    (dest_x, dest_y) in self._chip_info):
                opposite_link_id = (link + 3) % 6
                links.append(Link(
                    chip_info.x, chip_info.y, link, dest_x, dest_y,
                    opposite_link_id, opposite_link_id))
        router = Router(
            links=links,
            n_available_multicast_entries=(
                chip_info.n_free_multicast_routing_entries))

        # Create the chip's SDRAM object
        sdram_size = chip_info.largest_free_sdram_block
        if (self._max_sdram_size is not None and
                sdram_size > self._max_sdram_size):
            sdram_size = self._max_sdram_size
        sdram = SDRAM(size=sdram_size)

        # Create the chip
        chip = Chip(
            x=chip_info.x, y=chip_info.y, processors=processors,
            router=router, sdram=sdram,
            ip_address=chip_info.ethernet_ip_address,
            nearest_ethernet_x=chip_info.nearest_ethernet_x,
            nearest_ethernet_y=chip_info.nearest_ethernet_y)

        return chip

    def _receive_p2p_data(self, scp_read_response):
        self._p2p_column_data.append(
            (scp_read_response.data, scp_read_response.offset))

    def _receive_chip_info(self, scp_read_chip_info_response):
        chip_info = scp_read_chip_info_response.chip_info
        self._chip_info[chip_info.x, chip_info.y] = chip_info

    def _receive_error(self, request, exception, tracebackinfo):

        # If we get an SCPReadLinkRequest with a
        # SpinnmanUnexpectedResponseCodeException, this is a failed link
        # and so can be ignored
        if (not isinstance(request, SCPReadLinkRequest) or
                not isinstance(
                    exception,
                    exceptions.SpinnmanUnexpectedResponseCodeException)):
            AbstractProcess._receive_error(self, request, exception,
                                           tracebackinfo)

    def get_machine_details(self, boot_x, boot_y, width, height):

        # Get the P2P table - 8 entries are packed into each 32-bit word
        p2p_column_bytes = P2PTable.get_n_column_bytes(height)
        for column in range(width):
            offset = P2PTable.get_column_offset(column)
            self._send_request(
                SCPReadMemoryRequest(
                    x=boot_x, y=boot_y,
                    base_address=(
                        constants.ROUTER_REGISTER_P2P_ADDRESS + offset),
                    size=p2p_column_bytes),
                self._receive_p2p_data)
        self._finish()
        self.check_for_error()
        p2p_table = P2PTable(width, height, self._p2p_column_data)

        # Get the chip information for each chip
        for (x, y) in p2p_table.iterchips():
            self._send_request(
                SCPChipInfoRequest(x, y), self._receive_chip_info)
        self._finish()
        try:
            self.check_for_error()
        except:

            # Ignore errors so far, as any error here just means that a chip
            # is down that wasn't marked as down
            pass

        # Warn about unexpected missing chips
        for (x, y) in p2p_table.iterchips():
            if (x, y) not in self._chip_info:
                logger.warn("Chip {}, {} was expected but didn't reply".format(
                    x, y))

        # Build a Machine
        chips = [
            self._make_chip(width, height, chip_info)
            for chip_info in sorted(
                self._chip_info.values(),
                key=lambda chip: (chip.x, chip.y))
            if (chip_info.x, chip_info.y) not in self._ignore_chips]
        machine = Machine(chips, boot_x, boot_y)

        return machine

    def get_chip_info(self):
        """ Get the chip information for the machine.  Note that\
            get_machine_details must have been called first
        """
        return self._chip_info



from enum import Enum


class DiagnosticFilterPacketType(Enum):
    """ Packet type flags for the diagnostic filters.
        Note that only one has to match for the counter to be incremented
    """
    MULTICAST = (0, "Packet is multicast")
    POINT_TO_POINT = (1, "Packet is point-to-point")
    NEAREST_NEIGHBOUR = (2, "Packet is nearest-neighbour")
    FIXED_ROUTE = (3, "Packet is fixed-route")

    def __new__(cls, value, doc=""):
        obj = object.__new__(cls)
        obj._value_ = value
        return obj

    def __init__(self, value, doc=""):
        self._value_ = value
        self.__doc__ = doc



from spinnman.exceptions import SpinnmanIOException
from spinnman.exceptions import SpinnmanTimeoutException
from spinnman.connections.abstract_classes.abstract_connection \
    import AbstractConnection

import platform
import subprocess
import socket
import select


class UDPConnection(AbstractConnection):

    def __init__(self, local_host=None, local_port=None, remote_host=None,
                 remote_port=None):
        """
        :param local_host: The local host name or ip address to bind to.\
                    If not specified defaults to bind to all interfaces,\
                    unless remote_host is specified, in which case binding is\
                    _done to the ip address that will be used to send packets
        :type local_host: str or None
        :param local_port: The local port to bind to, between 1025 and 65535.\
                    If not specified, defaults to a random unused local port
        :type local_port: int
        :param remote_host: The remote host name or ip address to send packets\
                    to.  If not specified, the socket will be available for\
                    listening only, and will throw and exception if used for\
                    sending
        :type remote_host: str or None
        :param remote_port: The remote port to send packets to.  If\
                    remote_host is None, this is ignored.  If remote_host is\
                    specified, this must also be specified for the connection\
                    to allow sending
        :raise spinnman.exceptions.SpinnmanIOException: If there is an error\
                    setting up the communication channel
        """

        self._socket = None
        try:

            # Create a UDP Socket
            self._socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        except Exception as exception:
            raise SpinnmanIOException(
                "Error setting up socket: {}".format(exception))

        # Get the port to bind to locally
        local_bind_port = 0
        if local_port is not None:
            local_bind_port = int(local_port)

        # Get the host to bind to locally
        local_bind_host = ""
        if local_host is not None:
            local_bind_host = str(local_host)

        try:
            # Bind the socket
            self._socket.bind((local_bind_host, local_bind_port))

        except Exception as exception:
            raise SpinnmanIOException(
                "Error binding socket to {}:{}: {}".format(
                    local_bind_host, local_bind_port, exception))

        # Mark the socket as non-sending, unless the remote host is
        # specified - send requests will then cause an exception
        self._can_send = False
        self._remote_ip_address = None
        self._remote_port = None

        # Get the host to connect to remotely
        if remote_host is not None and remote_port is not None:
            self._can_send = True
            self._remote_port = remote_port

            try:
                self._remote_ip_address = socket.gethostbyname(remote_host)
            except Exception as exception:
                raise SpinnmanIOException(
                    "Error getting ip address for {}: {}".format(
                        remote_host, exception))

            try:
                self._socket.connect((self._remote_ip_address, remote_port))
            except Exception as exception:
                raise SpinnmanIOException(
                    "Error connecting to {}:{}: {}".format(
                        self._remote_ip_address, remote_port, exception))

        # Get the details of where the socket is connected
        self._local_ip_address = None
        self._local_port = None
        try:
            self._local_ip_address, self._local_port =\
                self._socket.getsockname()

            # Ensure that a standard address is used for the INADDR_ANY
            # hostname
            if self._local_ip_address is None or self._local_ip_address == "":
                self._local_ip_address = "0.0.0.0"
        except Exception as exception:
            raise SpinnmanIOException("Error querying socket: {}".format(
                exception))

        # Set a general timeout on the socket
        self._socket.settimeout(1.0)

    def is_connected(self):
        """ See\
            :py:meth:`spinnman.connections.AbstractConnection.abstract_connection.is_connected`
        """

        # If this is not a sending socket, it is not connected
        if not self._can_send:
            return False

        # check if machine is active and on the network
        pingtimeout = 5
        while pingtimeout > 0:

            # Start a ping process
            process = None
            if platform.platform().lower().startswith("windows"):
                process = subprocess.Popen(
                    "ping -n 1 -w 1 " + self._remote_ip_address,
                    shell=True, stdout=subprocess.PIPE)
            else:
                process = subprocess.Popen(
                    "ping -c 1 -W 1 " + self._remote_ip_address,
                    shell=True, stdout=subprocess.PIPE)
            process.wait()

            if process.returncode == 0:

                # ping worked
                return True
            else:
                pingtimeout -= 1

        # If the ping fails this number of times, the host cannot be contacted
        return False

    @property
    def local_ip_address(self):
        """ The local IP address to which the connection is bound.

        :return: The local ip address as a dotted string e.g. 0.0.0.0
        :rtype: str
        :raise None: No known exceptions are thrown
        """
        return self._local_ip_address

    @property
    def local_port(self):
        """ The local port to which the connection is bound.

        :return: The local port number
        :rtype: int
        :raise None: No known exceptions are thrown
        """
        return self._local_port

    @property
    def remote_ip_address(self):
        """ The remote ip address to which the connection is connected.

        :return: The remote ip address as a dotted string, or None if not\
                    connected remotely
        :rtype: str
        """
        return self._remote_ip_address

    @property
    def remote_port(self):
        """ The remote port to which the connection is connected.

        :return: The remote port, or None if not connected remotely
        :rtype: int
        """
        return self._remote_port

    def receive(self, timeout=None):
        """ Receive data from the connection

        :param timeout: The timeout, or None to wait forever
        :type timeout: None
        :return: The data received as a bytestring
        :rtype: str
        :raise SpinnmanTimeoutException: If a timeout occurs before any data\
                    is received
        :raise SpinnmanIOException: If an error occurs receiving the data
        """
        try:
            self._socket.settimeout(timeout)
            return self._socket.recv(300)
        except socket.timeout:
            raise SpinnmanTimeoutException("receive", timeout)
        except Exception as e:
            raise SpinnmanIOException(str(e))

    def receive_with_address(self, timeout=None):
        """ Receive data from the connection along with the address where the\
            data was received from

        :param timeout: The timeout, or None to wait forever
        :type timeout: None
        :return: A tuple of the data received and a tuple of the\
                (address, port) received from
        :rtype: str, (str, int)
        :raise SpinnmanTimeoutException: If a timeout occurs before any data\
                    is received
        :raise SpinnmanIOException: If an error occurs receiving the data
        """
        try:
            self._socket.settimeout(timeout)
            return self._socket.recvfrom(300)
        except socket.timeout:
            raise SpinnmanTimeoutException("receive", timeout)
        except Exception as e:
            raise SpinnmanIOException(str(e))

    def send(self, data):
        """ Send data down this connection

        :param data: The data to be sent
        :type data: str
        :raise SpinnmanIOException: If there is an error sending the data
        """
        if not self._can_send:
            raise SpinnmanIOException(
                "Remote host and/or port not set - data cannot be sent with"
                " this connection")
        try:
            self._socket.send(data)
        except Exception as e:
            raise SpinnmanIOException(str(e))

    def send_to(self, data, address):
        """ Send data down this connection

        :param data: The data to be sent as a bytestring
        :type data: str
        :param address: A tuple of (address, port) to send the data to
        :type address: (str, int)
        :raise SpinnmanIOException: If there is an error sending the data
        """
        try:
            self._socket.sendto(data, address)
        except Exception as e:
            raise SpinnmanIOException(str(e))

    def close(self):
        """ See\
            :py:meth:`spinnman.connections.abstract_connection.AbstractConnection.close`
        """
        try:
            self._socket.shutdown(socket.SHUT_WR)
        except:
            pass
        self._socket.close()

    def is_ready_to_receive(self, timeout=0):
        return len(select.select([self._socket], [], [], timeout)[0]) == 1

    def __repr__(self):
        return \
            "UDPConnection(local_host={}, local_port={}, remote_host={},"\
            "remote_port={})".format(
                self.local_ip_address, self.local_port,
                self.remote_ip_address, self.remote_port)



from spinnman.messages.eieio.command_messages.eieio_command_message\
    import EIEIOCommandMessage
from spinnman import constants
from spinnman.messages.eieio.command_messages.eieio_command_header\
    import EIEIOCommandHeader


class StopRequests(EIEIOCommandMessage):
    """ Packet used in the context of buffering input for the host computer to\
        signal to the SpiNNaker system that to stop sending\
        "SpinnakerRequestBuffers" packet
    """
    def __init__(self):
        EIEIOCommandMessage.__init__(
            self, EIEIOCommandHeader(
                constants.EIEIO_COMMAND_IDS.STOP_SENDING_REQUESTS.value))



from spinnman.messages.scp.impl.scp_sdram_alloc_request import \
    SCPSDRAMAllocRequest
from spinnman.processes.abstract_multi_connection_process \
    import AbstractMultiConnectionProcess


class MallocSDRAMProcess(AbstractMultiConnectionProcess):

    def __init__(self, connection_selector):
        AbstractMultiConnectionProcess.__init__(self, connection_selector)
        self._base_address = None

    def handle_sdram_alloc_response(self, response):
        self._base_address = response.base_address

    def malloc_sdram(self, x, y, size, app_id, tag):
        # Allocate space in the sdram space
        self._send_request(SCPSDRAMAllocRequest(x, y, app_id, size, tag),
                           self.handle_sdram_alloc_response)
        self._finish()
        self.check_for_error()

    @property
    def base_address(self):
        return self._base_address



from spinnman.messages.scp.impl.scp_dpri_set_router_emergency_timeout_request \
    import SCPDPRISetRouterEmergencyTimeoutRequest
from spinnman.processes.abstract_multi_connection_process \
    import AbstractMultiConnectionProcess


class SetDPRIRouterEmergencyTimeoutProcess(AbstractMultiConnectionProcess):

    def __init__(self, connection_selector):
        AbstractMultiConnectionProcess.__init__(self, connection_selector)

    def set_timeout(self, mantissa, exponent, core_subsets):
        for core_subset in core_subsets.core_subsets:
            for processor_id in core_subset.processor_ids:
                self._send_request(SCPDPRISetRouterEmergencyTimeoutRequest(
                    core_subset.x, core_subset.y, processor_id,
                    mantissa, exponent))
        self._finish()
        self.check_for_error()



import struct

from spinnman.utilities import utility_functions
from spinnman.messages.scp.enums.scp_dpri_packet_type_flags \
    import SCPDPRIPacketTypeFlags


class DPRIStatus(object):
    """ Represents a status information from dropped packet reinjection
    """

    def __init__(self, data, offset):
        """
        :param data: The data containing the information
        :type data: str
        :param offset: The offset in the data where the information starts
        :type offset: int
        """
        (self._router_timeout, self._router_emergency_timeout,
         self._n_dropped_packets, self._n_missed_dropped_packets,
         self._n_dropped_packet_overflows, self._n_reinjected_packets,
         self._n_link_dumps, self._n_processor_dumps,
         self._flags) = struct.unpack_from("<IIIIIIIII", data, offset)

    @property
    def router_timeout(self):
        """ The WAIT1 timeout value of the router in cycles
        """
        return utility_functions.get_router_timeout_value_from_byte(
            self._router_timeout)

    @property
    def router_emergency_timeout(self):
        """ The WAIT2 timeout value of the router in cycles
        """
        return utility_functions.get_router_timeout_value_from_byte(
            self._router_emergency_timeout)

    @property
    def n_dropped_packets(self):
        """ The number of packets dropped by the router and received by\
            the reinjector (may not fit in the queue though)
        """
        return self._n_dropped_packets

    @property
    def n_missed_dropped_packets(self):
        """ The number of times that when a dropped packet was read it was\
            found that another one or more packets had also been dropped,\
            but had been missed
        """
        return self._n_missed_dropped_packets

    @property
    def n_dropped_packet_overflows(self):
        """ Of the n_dropped_packets received, how many were lost due to not\
            having enough space in the queue of packets to reinject
        """
        return self._n_dropped_packet_overflows

    @property
    def n_processor_dumps(self):
        """ The number of times that when a dropped packet was caused due to
        a processor failing to take the packet.

        :return: int
        """
        return self._n_processor_dumps

    @property
    def n_link_dumps(self):
        """ The number of times that when a dropped packet was caused due to
        a link failing to take the packet.

        :return: int
        """
        return self._n_link_dumps

    @property
    def n_reinjected_packets(self):
        """ Of the n_dropped_packets received, how many packets were\
            successfully reinjected
        """
        return self._n_reinjected_packets

    @property
    def is_reinjecting_multicast(self):
        """ True if reinjection of multicast packets is enabled
        """
        return self._flags & SCPDPRIPacketTypeFlags.MULTICAST.value != 0

    @property
    def is_reinjecting_point_to_point(self):
        """ True if reinjection of point-to-point packets is enabled
        """
        return self._flags & SCPDPRIPacketTypeFlags.POINT_TO_POINT.value != 0

    @property
    def is_reinjecting_nearest_neighbour(self):
        """ True if reinjection of nearest neighbour packets is enabled
        """
        return (self._flags &
                SCPDPRIPacketTypeFlags.NEAREST_NEIGHBOUR.value != 0)

    @property
    def is_reinjecting_fixed_route(self):
        """ True if reinjection of fixed-route packets is enabled
        """
        return self._flags & SCPDPRIPacketTypeFlags.FIXED_ROUTE.value != 0



from enum import Enum
from collections import namedtuple
import struct

_SYSTEM_VARIABLES_BOOT_SIZE = 128


class _DataType(Enum):
    """ Enum for data types
    """
    BYTE = (1, "<B")
    SHORT = (2, "<H")
    INT = (4, "<I")
    LONG = (8, "<Q")
    BYTE_ARRAY = (16, "s")

    def __new__(cls, value, struct_code, doc=""):
        obj = object.__new__(cls)
        obj._value_ = value
        obj._struct_code = struct_code
        return obj

    def __init__(self, value, struct_code, doc=""):
        self._value_ = value
        self._struct_code = struct_code
        self.__doc__ = doc

    @property
    def struct_code(self):
        return self._struct_code


class _Definition(namedtuple("_Definition",
                             "offset, data_type, default, array_size, doc")):
    """
    helper class that contains a definition for a variable.
    """

    def __new__(cls, data_type, offset, default=0, array_size=None, doc=""):
        return super(_Definition, cls).__new__(
            cls, offset, data_type, default, array_size, doc)


class SystemVariableDefinition(Enum):
    """ Defines the system variables available
    """

    y = _Definition(
        _DataType.BYTE, offset=0, doc="The y-coordinate of the chip")
    x = _Definition(
        _DataType.BYTE, offset=0x01, doc="The x-coordinate of the chip")
    y_size = _Definition(
        _DataType.BYTE, offset=0x02,
        doc="The number of chips in the y-dimension")
    x_size = _Definition(
        _DataType.BYTE, offset=0x03,
        doc="The number of chips in the x-dimension")
    debug_y = _Definition(
        _DataType.BYTE, offset=0x04,
        doc="The y-coordinate of the chip to send debug messages to")
    debug_x = _Definition(
        _DataType.BYTE, offset=0x05,
        doc="The x-coordinate of the chip to send debug messages to")
    is_peer_to_peer_available = _Definition(
        _DataType.BYTE, offset=0x06,
        doc="Indicates if peer-to-peer is working on the chip")
    nearest_neighbour_last_id = _Definition(
        _DataType.BYTE, offset=0x07,
        doc="The last id used in nearest neighbour transaction")
    nearest_ethernet_y = _Definition(
        _DataType.BYTE, offset=0x08,
        doc="The x-coordinate of the nearest chip with Ethernet")
    nearest_ethernet_x = _Definition(
        _DataType.BYTE, offset=0x09,
        doc="The y-coordinate of the nearest chip with Ethernet")
    hardware_version = _Definition(
        _DataType.BYTE, offset=0x0a,
        doc="The version of the hardware in use")
    is_ethernet_available = _Definition(
        _DataType.BYTE, offset=0x0b,
        doc="Indicates if Ethernet is available on this chip")
    p2p_b_repeats = _Definition(
        _DataType.BYTE, offset=0x0c,
        doc="Number of times to send out P2PB packets")
    log_peer_to_peer_sequence_length = _Definition(
        _DataType.BYTE, offset=0x0d, default=4,
        doc="Log (base 2) of the peer-to-peer sequence length")
    clock_divisor = _Definition(
        _DataType.BYTE, offset=0x0e, default=0x33,
        doc="The clock divisors for system & router clocks")
    time_phase_scale = _Definition(
        _DataType.BYTE, offset=0x0f,
        doc="The time-phase scaling factor")
    clock_milliseconds = _Definition(
        _DataType.LONG, offset=0x10,
        doc="The time since startup in milliseconds")
    time_milliseconds = _Definition(
        _DataType.SHORT, offset=0x18,
        doc="The number of milliseconds in the current second")
    ltpc_period = _Definition(
        _DataType.SHORT, offset=0x1a)
    unix_timestamp = _Definition(
        _DataType.INT, offset=0x1c,
        doc="The time in seconds since midnight, 1st January 1970")
    router_time_phase_timer = _Definition(
        _DataType.INT, offset=0x20,
        doc="The router time-phase timer")
    cpu_clock_mhz = _Definition(
        _DataType.SHORT, offset=0x24, default=200,
        doc="The CPU clock frequency in MHz")
    sdram_clock_frequency_mhz = _Definition(
        _DataType.SHORT, offset=0x26, default=130,
        doc="The SDRAM clock frequency in MHz")
    nearest_neighbour_forward = _Definition(
        _DataType.BYTE, offset=0x28, default=0x3F,
        doc="Nearest-Neighbour forward parameter")
    nearest_neighbour_retry = _Definition(
        _DataType.BYTE, offset=0x29,
        doc="Nearest-Neighbour retry parameter")
    link_peek_timeout_microseconds = _Definition(
        _DataType.BYTE, offset=0x2a, default=100,
        doc="The link peek/poke timeout in microseconds")
    led_half_period_10_ms = _Definition(
        _DataType.BYTE, offset=0x2b, default=1,
        doc="The LED half-period in 10 ms units, or 1 to show load")
    netinit_bc_wait_time = _Definition(
        _DataType.BYTE, offset=0x2c, default=50,
        doc="The time to wait after last BC during network initialisation"
            " in 10 ms units")
    netinit_phase = _Definition(
        _DataType.BYTE, offset=0x2d, default=0,
        doc="The phase of boot process (see enum netinit_phase_e)")
    p2p_root_y = _Definition(
        _DataType.BYTE, offset=0x2e, default=0,
        doc="The y-coordinate of the chip from which the system was booted")
    p2p_root_x = _Definition(
        _DataType.BYTE, offset=0x2f, default=0,
        doc="The x-coordinate of the chip from which the system was booted")
    led_0 = _Definition(
        _DataType.INT, offset=0x30, default=0x1,
        doc="The first part of the LED definitions")
    led_1 = _Definition(
        _DataType.INT, offset=0x34, default=0,
        doc="The last part of the LED definitions")
    padding_1 = _Definition(
        _DataType.INT, offset=0x38, default=0,
        doc="A word of padding")
    random_seed = _Definition(
        _DataType.INT, offset=0x3c,
        doc="The random seed")
    is_root_chip = _Definition(
        _DataType.BYTE, offset=0x40,
        doc="Indicates if this is the root chip")
    n_shared_message_buffers = _Definition(
        _DataType.BYTE, offset=0x41, default=7,
        doc="The number of shared message buffers")
    nearest_neighbour_delay_us = _Definition(
        _DataType.BYTE, offset=0x42, default=10,
        doc="The delay between nearest-neighbour packets in microseconds")
    software_watchdog_count = _Definition(
        _DataType.BYTE, offset=0x43, default=3,
        doc="The number of watch dog timeouts before an error is raised")
    padding_2 = _Definition(
        _DataType.INT, offset=0x44, default=0,
        doc="A word of padding")
    system_ram_heap_address = _Definition(
        _DataType.INT, offset=0x48, default=1024,
        doc="The base address of the system SDRAM heap")
    sdram_heap_address = _Definition(
        _DataType.INT, offset=0x4c, default=0,
        doc="The base address of the user SDRAM heap")
    iobuf_size = _Definition(
        _DataType.INT, offset=0x50, default=16384,
        doc="The size of the iobuf buffer in bytes")
    system_sdram_bytes = _Definition(
        _DataType.INT, offset=0x54, default=8388608,
        doc="The size of the system SDRAM in bytes")
    system_buffer_words = _Definition(
        _DataType.INT, offset=0x58, default=32768,
        doc="The size of the system buffer in words")
    boot_signature = _Definition(
        _DataType.INT, offset=0x5c,
        doc="The boot signature")
    nearest_neighbour_memory_pointer = _Definition(
        _DataType.INT, offset=0x60,
        doc="The memory pointer for nearest neighbour global operations")
    lock = _Definition(
        _DataType.BYTE, offset=0x64, default=0,
        doc="The lock")
    links_available = _Definition(
        _DataType.BYTE, offset=0x65, default=0x3f,
        doc="Bit mask (6 bits) of links enabled")
    last_biff_id = _Definition(
        _DataType.BYTE, offset=0x66, default=0,
        doc="Last ID used in BIFF packet")
    board_test_flags = _Definition(
        _DataType.BYTE, offset=0x67,
        doc="Board testing flags")
    shared_message_first_free_address = _Definition(
        _DataType.INT, offset=0x68,
        doc="Pointer to the first free shared message buffer")
    shared_message_count_in_use = _Definition(
        _DataType.SHORT, offset=0x6c,
        doc="The number of shared message buffers in use")
    shared_message_maximum_used = _Definition(
        _DataType.SHORT, offset=0x6e,
        doc="The maximum number of shared message buffers used")
    user_temp_0 = _Definition(
        _DataType.INT, offset=0x70,
        doc="The first user variable")
    user_temp_1 = _Definition(
        _DataType.INT, offset=0x74,
        doc="The second user variable")
    user_temp_2 = _Definition(
        _DataType.INT, offset=0x78,
        doc="The third user variable")
    user_temp_4 = _Definition(
        _DataType.INT, offset=0x7c,
        doc="The fourth user variable")
    status_map = _Definition(
        _DataType.BYTE_ARRAY, offset=0x80, array_size=20,
        default=bytes(bytearray(20)),
        doc="The status map set during SCAMP boot")
    physical_to_virtual_core_map = _Definition(
        _DataType.BYTE_ARRAY, offset=0x94, array_size=20,
        default=bytes(bytearray(20)),
        doc="The physical core id to virtual core id map")
    virtual_to_physical_core_map = _Definition(
        _DataType.BYTE_ARRAY, offset=0xa8, array_size=20,
        default=bytes(bytearray(20)),
        doc="The virtual core id to physical core id map")
    n_working_cores = _Definition(
        _DataType.BYTE, offset=0xbc,
        doc="The number of working cores")
    n_scamp_working_cores = _Definition(
        _DataType.BYTE, offset=0xbd,
        doc="The number of SCAMP working cores")
    padding_3 = _Definition(
        _DataType.SHORT, offset=0xbe,
        doc="A short of padding")
    sdram_base_address = _Definition(
        _DataType.INT, offset=0xc0,
        doc="The base address of SDRAM")
    system_ram_base_address = _Definition(
        _DataType.INT, offset=0xc4,
        doc="The base address of System RAM")
    system_sdram_base_address = _Definition(
        _DataType.INT, offset=0xc8,
        doc="The base address of System SDRAM")
    cpu_information_base_address = _Definition(
        _DataType.INT, offset=0xcc,
        doc="The base address of the cpu information blocks")
    system_sdram_heap_address = _Definition(
        _DataType.INT, offset=0xd0,
        doc="The base address of the system SDRAM heap")
    router_table_copy_address = _Definition(
        _DataType.INT, offset=0xd4,
        doc="The address of the copy of the routing tables")
    peer_to_peer_hop_table_address = _Definition(
        _DataType.INT, offset=0xd8,
        doc="The address of the peer-to-peer hop tables")
    allocated_tag_table_address = _Definition(
        _DataType.INT, offset=0xdc,
        doc="The address of the allocated tag table")
    first_free_router_entry = _Definition(
        _DataType.SHORT, offset=0xe0,
        doc="The id of the first free router entry")
    n_active_peer_to_peer_addresses = _Definition(
        _DataType.SHORT, offset=0xe2,
        doc="The number of active peer-to-peer addresses")
    app_data_table_address = _Definition(
        _DataType.INT, offset=0xe4,
        doc="The address of the application data table")
    shared_message_buffer_address = _Definition(
        _DataType.INT, offset=0xe8,
        doc="The address of the shared message buffers")
    monitor_mailbox_flags = _Definition(
        _DataType.INT, offset=0xec,
        doc="The monitor incoming mailbox flags")
    ethernet_ip_address = _Definition(
        _DataType.BYTE_ARRAY, offset=0xf0, array_size=4,
        default=bytes(bytearray(4)),
        doc="The ip address of the chip")
    fixed_route_copy = _Definition(
        _DataType.INT, offset=0xf4,
        doc="A (virtual) copy of the router FR register")
    board_info = _Definition(
        _DataType.INT, offset=0xf8,
        doc="A pointer to the board information structure")
    padding_4 = _Definition(
        _DataType.INT, offset=0xfc,
        doc="A word of padding")

    def __init__(self, offset, data_type, default, array_size, doc):
        """

        :param data_type: The data type of the variable
        :type data_type: :py:class:`_DataType`
        :param offset: The offset from the start of the system variable\
                    structure where the variable is found
        :type offset: int
        :param default: The default value assigned to the variable\
                    if not overridden
        :type default: int
        :param array_size: The length of the array, or None if not an array
        :type array_size: int
        """
        self._data_type = data_type
        self._offset = offset
        self._default = default
        self._array_size = array_size
        self.__doc__ = doc

    @property
    def data_type(self):
        return self._data_type

    @property
    def array_size(self):
        return self._array_size

    @property
    def offset(self):
        return self._offset

    @property
    def default(self):
        return self._default


class SystemVariableBootValues(object):
    """ Default values of the system variables that get passed to SpiNNaker\
        during boot
    """

    def __init__(self, hardware_version=None, led_0=None):

        # Create a dict of variable values
        self._values = dict()
        for variable in SystemVariableDefinition:
            self._values[variable] = variable.default

        if hardware_version is not None:
            self._values[SystemVariableDefinition.hardware_version] =\
                hardware_version
        if led_0 is not None:
            self._values[SystemVariableDefinition.led_0] = led_0

    def set_value(self, system_variable_definition, value):
        self._values[system_variable_definition] = value

    @property
    def bytestring(self):
        data = b""
        for sys_var in SystemVariableDefinition:
            data += struct.pack(sys_var.data_type.struct_code,
                                self._values[sys_var])
        return data


spinnaker_boot_values = {
    1: SystemVariableBootValues(
        hardware_version=1, led_0=0x00076104),
    2: SystemVariableBootValues(
        hardware_version=2, led_0=0x00006103),
    3: SystemVariableBootValues(
        hardware_version=3, led_0=0x00000502),
    4: SystemVariableBootValues(
        hardware_version=4, led_0=0x00000001),
    5: SystemVariableBootValues(
        hardware_version=5, led_0=0x00000001)}



from six import add_metaclass

from spinnman.connections.abstract_classes.abstract_connection \
    import AbstractConnection
from spinn_utilities.abstract_base import AbstractBase, abstractmethod


@add_metaclass(AbstractBase)
class AbstractEIEIOSender(AbstractConnection):
    """ A sender of EIEIO messages
    """

    __slots__ = ()

    @abstractmethod
    def send_eieio_message(self, eieio_message):
        """ Sends an EIEIO message down this connection

        :param eieio_message: The eieio message to be sent
        :type eieio_message: \
                    :py:class:`spinnman.messages.eieio.abstract_messages.abstract_eieio_message.AbstractEIEIOMessage`
        :raise spinnman.exceptions.SpinnmanIOException: If there is an error\
                    sending the message
        """



from spinnman.exceptions import SpinnmanInvalidParameterException
from time import localtime
from time import asctime
import struct
import re
from spinnman import exceptions


class VersionInfo(object):
    """ Decodes SC&MP/SARK version information as returned by the SVER command
    """

    def __init__(self, version_data, offset=0):
        """
        :param version_data: bytes from an SCP packet containing version\
                    information
        :param offset: the offset in the bytes from an SCP packet containing
                       version information
        :type version_data: bytearray
        :raise spinnman.exceptions.SpinnmanInvalidParameterException: If the\
                    message does not contain valid version information
        """
        (self._p, self._physical_cpu_id, self._y, self._x, _,
            version_no, self._build_date) = struct.unpack_from(
                "<BBBBHHI", buffer(version_data), offset)

        version_data = version_data[offset + 12:-1].decode("utf-8")

        if version_no < 0xFFFF:
            try:
                self._version_number = (version_no // 100, version_no % 100, 0)
                self._name, self._hardware = version_data.split("/")
                self._version_string = version_data
            except ValueError as exception:
                raise SpinnmanInvalidParameterException(
                    "version_data", version_data,
                    "Incorrect format: {}".format(exception))
        else:
            name_hardware, _, version = version_data.partition("\0")
            self._version_string = version
            matches = re.match("(\d+)\.(\d+)\.(\d+)", version)
            if matches is None:
                raise exceptions.SpinnmanInvalidParameterException(
                    "version", version, "Cannot be parsed")
            self._version_number = tuple(map(int, matches.group(1, 2, 3)))
            self._name, self._hardware = name_hardware.rstrip("\0").split("/")

    @property
    def name(self):
        """ The name of the software

        :return: The name
        :rtype: str
        """
        return self._name

    @property
    def version_number(self):
        """ The version number of the software

        :return: The version
        :rtype: float
        """
        return self._version_number

    @property
    def hardware(self):
        """ The hardware being run on

        :return: The hardware
        :rtype: str
        """
        return self._hardware

    @property
    def x(self):
        """ The x-coordinate of the chip where the information was obtained

        :return: the x-coordinate
        :rtype: int
        """
        return self._x

    @property
    def y(self):
        """ The y-coordinate of the chip where the information was obtained

        :return: The y-coordinate
        :rtype: int
        """
        return self._y

    @property
    def p(self):
        """ The processor id of the processor where the information was\
                    obtained

        :return: the processor id
        :rtype: int
        """
        return self._p

    @property
    def build_date(self):
        """ The build date of the software

        :return: The number of seconds since 1st January 1970
        :rtype: long
        """
        return self._build_date

    @property
    def version_string(self):
        """ The version information as text

        :return: The version information
        :rtype: str
        """
        return self._version_string

    def __str__(self):
        return "[Version: {} {} at {}:{}:{}:{} (built {})]".format(
            self._name, self._version_string, self._hardware, self._x, self._y,
            self._p, asctime(localtime(self._build_date)))



from enum import Enum


class EIEIOPrefix(Enum):
    """ Possible prefixing of keys in EIEIO packets
    """
    LOWER_HALF_WORD = (0, "apply prefix on lower half of the word")
    UPPER_HALF_WORD = (1, "apply prefix on top half of the word")

    def __new__(cls, value, doc=""):
        obj = object.__new__(cls)
        obj._value_ = value
        return obj

    def __init__(self, value, doc=""):
        self._value_ = value
        self.__doc__ = doc



from spinnman.messages.scp.abstract_messages.abstract_scp_response\
    import AbstractSCPResponse
from spinnman.model.dpri_status import DPRIStatus
from spinnman.exceptions import SpinnmanUnexpectedResponseCodeException
from spinnman.messages.scp.enums.scp_result import SCPResult
from spinnman.messages.scp.enums.scp_command import SCPCommand


class SCPDPRIGetStatusResponse(AbstractSCPResponse):
    """ An SCP response to a request for the dropped packet reinjection status
    """

    def __init__(self):
        """
        """
        AbstractSCPResponse.__init__(self)
        self._dpri_status = None

    def read_data_bytestring(self, data, offset):
        """ See\
            :py:meth:`spinnman.messages.scp.abstract_scp_response.AbstractSCPResponse.read_data_bytestring`
        """
        result = self.scp_response_header.result
        if result != SCPResult.RC_OK:
            raise SpinnmanUnexpectedResponseCodeException(
                "Get dropped packet reinjection status", SCPCommand.CMD_DPRI,
                result.name)
        self._dpri_status = DPRIStatus(data, offset)

    @property
    def dpri_status(self):
        return self._dpri_status



class MulticastMessage(object):
    """ A SpiNNaker Multicast message
    """

    def __init__(self, key, payload=None):
        """A multicast message has a key (determining the target locations) and
        an optional payload.

        :param key: The key of the packet
        :type key: int
        :param payload: The optional payload of the packet
        :type payload: int
        :raise None: No known exceptions are raised
        """
        self._key = key
        self._payload = payload

    @property
    def key(self):
        """ The key of the packet

        :return: The key
        :rtype: int
        """
        return self._key

    @property
    def payload(self):
        """ The payload of the packet if there is one

        :return: The payload, or None if there is no payload
        :rtype: int
        """
        return self._payload



from six import add_metaclass

from spinn_utilities.abstract_base import AbstractBase, abstractmethod
from spinnman.connections.abstract_classes.abstract_connection \
    import AbstractConnection


@add_metaclass(AbstractBase)
class AbstractSCPReceiver(AbstractConnection):
    """ A receiver of SCP messages
    """

    __slots__ = ()

    @abstractmethod
    def is_ready_to_receive(self, timeout=0):
        """ Determines if there is an SCP packet to be read without blocking

        :param timeout: The time to wait before returning if the connection\
                is not ready
        :type timeout: int
        :return: True if there is an SCP packet to be read
        :rtype: bool
        """
        pass

    @abstractmethod
    def receive_scp_response(self, timeout=1.0):
        """ Receives an SCP response from this connection.  Blocks\
            until a message has been received, or a timeout occurs.

        :param timeout: The time in seconds to wait for the message to arrive;\
                    if not specified, will wait forever, or until the\
                    connection is closed
        :type timeout: int
        :return: The SCP result, the sequence number, the data of the response\
                    and the offset at which the data starts (i.e. where the
                    SDP header starts)
        :rtype: :py:class: `spinnman.messages.scp.scp_result.SCPResult`,\
                    int, bytestring, int
        :raise spinnman.exceptions.SpinnmanIOException: If there is an error\
                    receiving the message
        :raise spinnman.exceptions.SpinnmanTimeoutException: If there is a\
                    timeout before a message is received
        """



"""
SCPReadFPGARegisterResponse
"""
import struct

from spinnman.messages.scp.abstract_messages.abstract_scp_bmp_reponse import \
    AbstractSCPBMPResponse
from spinnman.messages.scp.enums.scp_result import SCPResult
from spinnman.exceptions import SpinnmanUnexpectedResponseCodeException


class SCPReadFPGARegisterResponse(AbstractSCPBMPResponse):
    """ An SCP response to a request for the version of software running
    """

    def __init__(self):
        """
        """
        AbstractSCPBMPResponse.__init__(self)
        self._fpga_register = None

    def read_data_bytestring(self, data, offset):
        """ See\
            :py:meth:`spinnman.messages.scp.abstract_scp_response.AbstractSCPResponse.read_data_bytestring`
        """
        result = self.scp_response_header.result
        if result != SCPResult.RC_OK:
            raise SpinnmanUnexpectedResponseCodeException(
                "Read FPGA register", "CMD_LINK_READ", result.name)

        self._fpga_register = struct.unpack_from("<I", data, offset)[0]

    @property
    def fpga_register(self):
        """ The register information received

        :rtype: int
        """
        return self._fpga_register



from spinnman.messages.eieio.command_messages.eieio_command_message\
    import EIEIOCommandMessage
from spinnman.messages.eieio.command_messages.eieio_command_header\
    import EIEIOCommandHeader
from spinnman import constants
import struct


class SpinnakerRequestBuffers(EIEIOCommandMessage):
    """ Message used in the context of the buffering input mechanism which is\
        sent by the SpiNNaker system to the host computer to ask for more data\
        to inject during the simulation
    """
    def __init__(self, x, y, p, region_id, sequence_no, space_available):
        EIEIOCommandMessage.__init__(
            self, EIEIOCommandHeader(
                constants.EIEIO_COMMAND_IDS.SPINNAKER_REQUEST_BUFFERS.value))
        self._x = x
        self._y = y
        self._p = p
        self._region_id = region_id
        self._sequence_no = sequence_no
        self._space_available = space_available

    @property
    def x(self):
        return self._x

    @property
    def y(self):
        return self._y

    @property
    def p(self):
        return self._p

    @property
    def region_id(self):
        return self._region_id

    @property
    def sequence_no(self):
        return self._sequence_no

    @property
    def space_available(self):
        return self._space_available

    @staticmethod
    def get_min_packet_length():
        return 12

    @staticmethod
    def from_bytestring(command_header, data, offset):
        y, x, processor, region_id, sequence_no, space = struct.unpack_from(
            "<BBBxBBI", data, offset)
        p = (processor >> 3) & 0x1F
        return SpinnakerRequestBuffers(x, y, p, region_id & 0xF, sequence_no,
                                       space)

    @property
    def bytestring(self):
        return (super(SpinnakerRequestBuffers, self).bytestring + struct.pack(
            "<BBBxBBI", self._y, self._x, self._p << 3, self._region_id,
            self._sequence_no, self._space_available))



from six import add_metaclass

from spinn_utilities.abstract_base import AbstractBase, abstractmethod, \
    abstractproperty
from spinnman.connections.abstract_classes.abstract_connection \
    import AbstractConnection


@add_metaclass(AbstractBase)
class AbstractSCPSender(AbstractConnection):
    """ A sender of SCP messages
    """

    __slots__ = ()

    @abstractmethod
    def get_scp_data(self, scp_request):
        """ Returns the data of an SCP request as it would be sent down this\
            connection
        """

    @abstractmethod
    def send_scp_request(self, scp_request):
        """ Sends an SCP request down this connection

         Messages must have the following properties:

            * source_port is None or 7
            * source_cpu is None or 31
            * source_chip_x is None or 0
            * source_chip_y is None or 0

        tag in the message is optional - if not set the default set in the\
        constructor will be used.
        sequence in the message is optional - if not set (sequence number\
        last assigned + 1) % 65536 will be used

        :param scp_request: message packet to send
        :type scp_request:\
                    :py:class:`spinnman.messages.scp.abstract_scp_request.AbstractSCPRequest`
        :return: Nothing is returned
        :rtype: None
        :raise spinnman.exceptions.SpinnmanIOException: If there is an error\
                    sending the message
        """

    @abstractproperty
    def chip_x(self):
        """ The x-coordinate of the chip at which messages sent down this\
            connection will arrive at first

        :rtype: int
        """

    @abstractproperty
    def chip_y(self):
        """ The y-coordinate of the chip at which messages sent down this\
            connection will arrive at first

        :rtype: int
        """



from enum import Enum


class CPUState(Enum):
    """ SARK CPU States
    """
    DEAD = 0
    POWERED_DOWN = 1
    RUN_TIME_EXCEPTION = 2
    WATCHDOG = 3
    INITIALISING = 4
    READY = 5
    C_MAIN = 6
    RUNNING = 7
    SYNC0 = 8
    SYNC1 = 9
    PAUSED = 10
    FINISHED = 11
    CPU_STATE_12 = 12
    CPU_STATE_13 = 13
    CPU_STATE_14 = 14
    IDLE = 15

    def __new__(cls, value, doc=""):
        obj = object.__new__(cls)
        obj._value_ = value
        return obj

    def __init__(self, value, doc=""):
        self._value_ = value
        self.__doc__ = doc



import struct

from spinnman.messages.scp.abstract_messages.abstract_scp_response \
    import AbstractSCPResponse
from spinnman.messages.scp.enums.scp_result import SCPResult
from spinnman.exceptions import SpinnmanUnexpectedResponseCodeException


class SCPCountStateResponse(AbstractSCPResponse):
    """ An SCP response to a request for the number of cores in a given state
    """

    def __init__(self):
        """
        """
        super(SCPCountStateResponse, self).__init__()
        self._count = None

    def read_data_bytestring(self, data, offset):
        """ See\
            :py:meth:`spinnman.messages.scp.abstract_scp_response.AbstractSCPResponse.read_data_bytestring`
        """
        result = self.scp_response_header.result
        if result != SCPResult.RC_OK:
            raise SpinnmanUnexpectedResponseCodeException(
                "CountState", "CMD_SIGNAL", result.name)
        self._count = struct.unpack_from("<I", data, offset)[0]

    @property
    def count(self):
        """ The count of the number of cores with the requested state

        :rtype: int
        """
        return self._count



from spinnman.messages.scp.impl.scp_dpri_set_reinjection_packet_types \
    import SCPDPRISetReinjectionPacketTypesRequest
from spinnman.processes.abstract_multi_connection_process \
    import AbstractMultiConnectionProcess


class SetDPRIPacketTypesProcess(AbstractMultiConnectionProcess):

    def __init__(self, connection_selector):
        AbstractMultiConnectionProcess.__init__(self, connection_selector)

    def set_packet_types(self, packet_types, core_subsets):
        for core_subset in core_subsets.core_subsets:
            for processor_id in core_subset.processor_ids:
                self._send_request(SCPDPRISetReinjectionPacketTypesRequest(
                    core_subset.x, core_subset.y, processor_id, packet_types))
        self._finish()
        self.check_for_error()



"""
SCPReadFPGARegisterRequest
"""

# spinnman imports
from spinnman.messages.scp.abstract_messages.abstract_scp_bmp_request import \
    AbstractSCPBMPRequest
from spinnman.messages.scp.impl.scp_read_fpga_register_response import \
    SCPReadFPGARegisterResponse
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.scp_request_header import SCPRequestHeader


class SCPReadFPGARegisterRequest(AbstractSCPBMPRequest):
    """ Requests the data from a fpga's register
    """

    def __init__(self, fpga_num, register, board):
        """
        sets up a read fpga register request

        :param fpga_num: FPGA number (0, 1 or 2) to communicate with.
        :param register: Register address to read to (will be rounded down to
                the nearest 32-bit word boundary).
        :param board: which board to request the fpga register from
        :rtype: None
        """

        # check to stop people asking for none word aligned memory addresses
        # inverses all bits of a value, so is basically a inverse mask for the
        # value entered.
        arg1 = register & (~0x3)
        AbstractSCPBMPRequest.__init__(
            self, board,
            SCPRequestHeader(
                command=SCPCommand.CMD_LINK_READ),
            argument_1=arg1, argument_2=4, argument_3=fpga_num)

    def get_scp_response(self):
        return SCPReadFPGARegisterResponse()



import traceback


class SpinnmanException(Exception):
    """ Superclass of exceptions that occur when dealing with communication\
        with SpiNNaker
    """
    pass


class SpinnmanInvalidPacketException(SpinnmanException):
    """ An exception that indicates that a packet was not in the expected\
        format
    """

    def __init__(self, packet_type, problem):
        """
        :param packet_type: The type of packet expected
        :type packet_type: str
        :param problem: The problem with the packet
        :type problem: str
        """
        super(SpinnmanInvalidPacketException, self).__init__(
            "Invalid packet of type {} received: {}".format(
                packet_type, problem))
        self._packet_type = packet_type
        self._problem = problem

    @property
    def packet_type(self):
        """ The packet type
        """
        return self._packet_type

    @property
    def problem(self):
        """ The problem with the packet
        """
        return self._problem


class SpinnmanInvalidParameterException(SpinnmanException):
    """ An exception that indicates that the value of one of the parameters\
        passed was invalid
    """

    def __init__(self, parameter, value, problem):
        """
        :param parameter: The name of the parameter that is invalid
        :type parameter: str
        :param value: The value of the parameter that is invalid
        :type value: str
        :param problem: The problem with the parameter
        :type problem: str
        """
        super(SpinnmanInvalidParameterException, self).__init__(
            "Setting parameter {} to value {} is invalid: {}".format(
                parameter, value, problem))
        self._parameter = parameter
        self._value = value
        self._problem = problem

    @property
    def parameter(self):
        """ The parameter with an invalid value
        """
        return self._parameter

    @property
    def value(self):
        """ The value that is invalid
        """
        return self._value

    @property
    def problem(self):
        """ The problem with the parameter value
        """
        return self._problem


class SpinnmanInvalidParameterTypeException(SpinnmanException):
    """ An exception that indicates that the type of one of the parameters\
        passed was invalid
    """

    def __init__(self, parameter, param_type, problem):
        """
        :param parameter: The name of the parameter that is invalid
        :type parameter: str
        :param param_type: The type of the parameter that is invalid
        :type param_type: str
        :param problem: The problem with the parameter
        :type problem: str
        """
        super(SpinnmanInvalidParameterTypeException, self).__init__(
            "Parameter {} of type {} is invalid: {}".format(
                parameter, param_type, problem))
        self._parameter = parameter
        self._type = param_type
        self._problem = problem

    @property
    def parameter(self):
        """ The parameter with an invalid value
        """
        return self._parameter

    @property
    def type(self):
        """ The value that is invalid
        """
        return self._type

    @property
    def problem(self):
        """ The problem with the parameter value
        """
        return self._problem


class SpinnmanIOException(SpinnmanException):
    """ An exception that something went wrong with the underlying IO
    """

    def __init__(self, problem):
        """
        :param problem: The problem with the IO
        :type problem: str
        """
        super(SpinnmanIOException, self).__init__("IO Error: {}".format(
            problem))
        self._problem = problem

    @property
    def problem(self):
        """ The problem with IO
        """
        return self._problem


class SpinnmanTimeoutException(SpinnmanException):
    """ An exception that indicates that a timeout occurred before an operation
        could finish
    """

    def __init__(self, operation, timeout):
        """
        :param operation: The operation being performed
        :type operation: str
        :param timeout: The timeout value in seconds
        :type timeout: int
        """
        super(SpinnmanTimeoutException, self).__init__(
            "Operation {} timed out after {} seconds".format(
                operation, timeout))

        self._operation = operation
        self._timeout = timeout

    @property
    def operation(self):
        """ The operation that was performed
        """
        return self._operation

    @property
    def timeout(self):
        """ The timeout value in seconds
        """
        return self._timeout


class SpinnmanUnexpectedResponseCodeException(SpinnmanException):
    """ Indicate that a response code returned from the board was unexpected\
        for the current operation
    """

    def __init__(self, operation, command, response):
        """
        :param operation: The operation being performed
        :type operation: str
        :param command: The command being executed
        :type command: str
        :param response: The response received in error
        :type response: str
        """
        super(SpinnmanUnexpectedResponseCodeException, self).__init__(
            "Unexpected response {} while performing operation {} using"
            " command {}".format(response, operation, command))
        self._operation = operation
        self._command = command
        self._response = response

    @property
    def operation(self):
        """ The operation being performed
        """
        return self._operation

    @property
    def command(self):
        """ The command being executed
        """
        return self._command

    @property
    def response(self):
        """ The unexpected response
        """
        return self._response


class SpinnmanGenericProcessException(SpinnmanException):
    """Encapsulates exceptions from processes which communicate with some
    core/chip
    """
    def __init__(self, exception, tb, x, y, p):
        problem = \
            "\n     Received exception class: {} \n" \
            "     With message: {} \n" \
            "     When sending to {}:{}:{}\n" \
            "     Stack trace: {}\n"\
            .format(exception.__class__.__name__, exception.message, x, y, p,
                    traceback.format_exc(tb))
        SpinnmanException.__init__(self, problem)

        self._stored_exception = exception

    @property
    def exception(self):
        return self._stored_exception


class SpinnmanUnsupportedOperationException(SpinnmanException):
    """ An exception that indicates that the given operation is not supported
    """

    def __init__(self, operation):
        """
        :param operation: The operation being requested
        :type operation: str
        """
        super(SpinnmanUnsupportedOperationException, self).__init__(
            "Operation {} is not supported".format(operation))
        self._operation = operation

    @property
    def operation(self):
        """ The unsupported operation requested
        """
        return self._operation


class SpinnmanEIEIOPacketParsingException(SpinnmanException):
    """ Unable to complete the parsing of the EIEIO packet received.
    The routine used is invalid or the content of the packet is invalid
    """

    def __init__(self, parsing_format, packet):
        super(SpinnmanEIEIOPacketParsingException, self).__init__(
            "The packet received is being parsed as an EIEIO {0:s} packet, "
            "but the content of the packet is invalid".format(parsing_format))
        self._packet = packet

    @property
    def packet(self):
        return self._packet



from enum import Enum


class RunTimeError(Enum):
    """ SARK Run time errors
    """
    NONE = 0
    RESET = 1
    UNDEF = 2
    SVC = 3
    PABT = 4
    DABT = 5
    IRQ = 6
    FIQ = 7
    VIC = 8
    ABORT = 9
    MALLOC = 10
    DIVBY0 = 11
    EVENT = 12
    SWERR = 13
    IOBUF = 14
    ENABLE = 15
    NULL = 16
    PKT = 17
    TIMER = 18
    API = 19
    SARK_VERSRION_INCORRECT = 20

    def __new__(cls, value, doc=""):
        obj = object.__new__(cls)
        obj._value_ = value
        return obj

    def __init__(self, value, doc=""):
        self._value_ = value
        self.__doc__ = doc



from spinnman.exceptions import SpinnmanInvalidParameterException
import struct


class EIEIOCommandHeader(object):
    """ EIEIO header for command packets
    """

    def __init__(self, command):
        if command < 0 or command >= 16384:
            raise SpinnmanInvalidParameterException(
                "command", command,
                "parameter command is outside the allowed range (0 to 16383)")
        self._command = command

    @property
    def command(self):
        return self._command

    @staticmethod
    def from_bytestring(data, offset):
        """ Read an eieio command header from a bytestring

        :param data: The bytestring to read the data from
        :type data: str
        :param offset: The offset where the valid data starts
        :type offset: int
        :return: an EIEIO command header
        :rtype:\
                    :py:class:`spinnman.messages.eieio.command_messages.eieio_command_header.EIEIOCommandHeader`
        :raise spinnman.exceptions.SpinnmanIOException: If there is an error\
                    reading from the reader
        :raise spinnman.exceptions.SpinnmanInvalidParameterException: If there\
                    is an error setting any of the values
        """
        command_header = struct.unpack_from("<H", data, offset)[0]
        command = command_header & 0x3FFF

        return EIEIOCommandHeader(command)

    @property
    def bytestring(self):
        """ Get a bytestring of the header

        :rtype: str
        """
        return struct.pack("<H", 0 << 15 | 1 << 14 | self._command)



"""
ADCInfo
"""

# spinnman imports
from spinnman import constants

# general imports
import struct


class ADCInfo(object):
    """
    container for the ADC data thats been retrieved from a fpga
    """

    def __init__(self, adc_data, offset):
        """
        :param adc_data: bytes from an SCP packet containing adc\
                    information
        :type adc_data: str
        :raise spinnman.exceptions.SpinnmanInvalidParameterException: If the\
                    message does not contain valid adc information
        """
        data = struct.unpack_from(
            "<"   # Little-endian
            "8H"  # uint16_t adc[8]
            "4h"  # int16_t t_int[4]
            "4h"  # int16_t t_ext[4]
            "4h"  # int16_t fan[4]
            "I"   # uint32_t warning
            "I",  # uint32_t shutdown
            adc_data, offset)

        self._voltage_1_2c = data[1] * constants.BMP_V_SCALE_2_5
        self._voltage_1_2b = data[2] * constants.BMP_V_SCALE_2_5
        self._voltage_1_2a = data[3] * constants.BMP_V_SCALE_2_5
        self._voltage_1_8 = data[4] * constants.BMP_V_SCALE_2_5
        self._voltage_3_3 = data[6] * constants.BMP_V_SCALE_3_3
        self._voltage_supply = data[7] * constants.BMP_V_SCALE_12
        self._temp_top = float(data[8]) * constants.BMP_TEMP_SCALE
        self._temp_btm = float(data[9]) * constants.BMP_TEMP_SCALE
        if data[12] != constants.BMP_MISSING_TEMP:
            self._temp_ext_0 = (float(data[12]) * constants.BMP_TEMP_SCALE)
        else:
            self._temp_ext_0 = None
        if data[13] != constants.BMP_MISSING_TEMP:
            self._temp_ext_1 = (float(data[13]) * constants.BMP_TEMP_SCALE)
        else:
            self._temp_ext_1 = None
        if data[16] != constants.BMP_MISSING_FAN:
            self._fan_0 = float(data[16])
        else:
            self._fan_0 = None
        if data[17] != constants.BMP_MISSING_FAN:
            self._fan_1 = float(data[17])
        else:
            self._fan_1 = None

    @property
    def voltage_1_2c(self):
        """
        property for voltage 1 2c
        """
        return self._voltage_1_2c

    @property
    def voltage_1_2b(self):
        """
        property for voltage 1 2b
        """
        return self._voltage_1_2b

    @property
    def voltage_1_2a(self):
        """
        property for voltage 1 2a
        """
        return self._voltage_1_2a

    @property
    def voltage_1_8(self):
        """
        property for voltage 1 8
        """
        return self._voltage_1_8

    @property
    def voltage_3_3(self):
        """
        property for voltage 3 3
        """
        return self._voltage_3_3

    @property
    def voltage_supply(self):
        """
        property for voltage supply
        """
        return self._voltage_supply

    @property
    def temp_top(self):
        """
        property for temp_top
        """
        return self._temp_top

    @property
    def temp_btm(self):
        """
        property for temp btm
        """
        return self._temp_btm

    @property
    def temp_ext_0(self):
        """
        property for temp ext 0
        """
        return self._temp_ext_0

    @property
    def temp_ext_1(self):
        """
        property for temp_ext_1
        """
        return self._temp_ext_1

    @property
    def fan_0(self):
        """
        property for fan 0
        """
        return self._fan_0

    @property
    def fan_1(self):
        """
        property for fan 1
        """
        return self._fan_1



""" Used to communicate with a SpiNNaker Board.  The main part of this package\
    is the :py:class:`spinnman.transceiver.Transceiver` class.  This can be\
    used to send and receive packets in various SpiNNaker formats, depending\
    on what connections are available.

    Functional Requirements
    =======================

        * Connect to and communicate with a machine using a number of\
          different connections.

        * Boot a machine with the expected version of the software.

            * If the machine is already booted but the version is not the\
              version expected, an exception will be thrown.

        * Check the version of the software which the machine is booted with.

        * Query the state of the machine to determine:

            * What the current state of the machine is in terms of the chips\
              and cores available, the SDRAM available on the chips and the\
              which links are available between which chips.

            * What external links to the host exist (and separately add the\
              discovered links to the set of links used to communicate with\
              the machine).

            * What is running on the machine and where, and what the current\
              status of those processes are.

            * How many cores are in a given state.

            * What is in the IOBUF buffers.

            * What the current routing entries for a given router are.

            * What the routing status counter values are.

        * Load application binaries on to the machine, either to individual\
          cores or via a "flood-fill" mechanism to multiple cores\
          simultaneously (which may be a subset of the cores on a subset of\
          the chips).

        * Write data to SDRAM, either on an individual chip, or via a\
          "flood-fill" mechanism to multiple chips simultaneously.

        * Send a signal to an application.

        * Read data from SDRAM on an individual chip.

        * Send and receive SpiNNaker packets where the connections allow this.

            * If no connection supports this packet type, an exception is\
              thrown.

            * The user should be able to select which connection is used.\
              Selection of a connection which does not support the traffic\
              type will also result in an exception.

        * Send and receive SCP and SDP packets where the connections allow\
          this.

            * If no connection supports the packet type, an exception is\
              thrown.

            * The user should be able to select which connection is used.\
              Selection of a connection which does not support the traffic\
              type will also result in an exception.

        * It should be possible to call any of the functions simultaneously,\
          including the same function more than once.

            * Where possible, multiple connections should be used to overlap\
              calls.

            * The functions should not return until they have confirmed that\
              any messages sent have been received, and any responses have\
              been received.

            * Functions should not respond with the result of a different\
              function.

            * Functions can further sub-divide the call into a number of\
              separate calls that can be divided across the available\
              connections, so long as the other requirements are met.

        * More than one machine can be connected to the same host.

            * Once the subset of connections has been worked out for each\
              machine, the operation of these machines should be independent.

    Use Cases
    =========

        * :py:meth:`~spinnman.transceiver.Transceiver.boot_board` and\
          :py:meth:`~spinnman.transceiver.Transceiver.get_scamp_version` are\
          used to ensure that the board is booted correctly before starting a\
          simulation.

        * :py:meth:`~spinnman.transceiver.Transceiver.get_machine_details` is\
          used to get a representation of the current state of the machine,\
          which is used to decide where executables are to be run on the board\
          for a particular simulation, where any external peripherals are\
          connected, and how messages between the executables and/or the\
          external peripherals are to be routed

        * :py:meth:`~spinnman.transceiver.Transceiver.write_memory` and\
          :py:meth:`~spinnman.transceiver.Transceiver.execute` are used to\
          write parameters and execute executables on the board

        * :py:meth:`~spinnman.transceiver.Transceiver.send_signal` is used to\
          send a signal which starts, stops or pauses a simulation.

        * :py:meth:`~spinnman.transceiver.Transceiver.get_core_status_count`\
          is used to determine if a simulation is complete or has gone into an\
          error state.

        * :py:meth:`~spinnman.transceiver.Transceiver.get_iobuf`,\
          :py:meth:`~spinnman.transceiver.Transceiver.get_cpu_information` and\
          :py:meth:`~spinnman.transceiver.Transceiver.get_router_diagnostics`\
          are used to diagnose a problem with a simulation

        * :py:meth:`~spinnman.transceiver.Transceiver.read_memory` is used\
          to read some statistics recorded in SDRAM after a simulation

"""



from spinnman.model.enums.diagnostic_filter_destination \
    import DiagnosticFilterDestination
from spinnman.model.enums.diagnostic_filter_source \
    import DiagnosticFilterSource
from spinnman.model.enums.diagnostic_filter_payload_status \
    import DiagnosticFilterPayloadStatus
from spinnman.model.enums.diagnostic_filter_default_routing_status \
    import DiagnosticFilterDefaultRoutingStatus
from spinnman.model.enums.diagnostic_filter_emergency_routing_status \
    import DiagnosticFilterEmergencyRoutingStatus
from spinnman.model.enums.diagnostic_filter_packet_type \
    import DiagnosticFilterPacketType

# Bit offsets of the various fields in the filter word
_PACKET_TYPE_OFFSET = 0
_EMERGENCY_ROUTE_OFFSET = 4
_EMERGENCY_ROUTE_MODE_OFFSET = 8
_DEFAULT_ROUTE_OFFSET = 10
_PAYLOAD_OFFSET = 12
_SOURCE_OFFSET = 14
_DESTINATION_OFFSET = 16
_ENABLE_INTERRUPT_OFFSET = 30


# Uses an enum to set flags in the filter word from a given offset
def _set_flags_in_word(word, enum_list, enum_type, offset):
    if enum_list is None:
        enum_values = list()
    else:
        enum_values = list(enum_list)
    if len(enum_values) == 0:
        enum_values = [value for value in enum_type]
    for enum_value in enum_values:
        word |= 1 << (enum_value.value + offset)
    return word


# Uses an enum to read flags in the filter word from a given offset
def _read_flags_from_word(word, enum_list, offset):
    flags = list()
    for enum_value in enum_list:
        if word & 1 << (enum_value.value + offset) != 0:
            flags.append(enum_value)
    return flags


class DiagnosticFilter(object):
    """ A router diagnostic counter filter, which counts packets passing
        through the router with certain properties.  The counter will be\
        incremented so long as the packet matches one of the values in each\
        field i.e. one of each of the destinations, sources, payload_statuses,\
        default_routing_statuses, emergency_routing_statuses and packet_types
    """

    def __init__(self, enable_interrupt_on_counter_event,
                 match_emergency_routing_status_to_incoming_packet,
                 destinations, sources, payload_statuses,
                 default_routing_statuses, emergency_routing_statuses,
                 packet_types):
        """
        :param enable_interrupt_on_counter_event: Indicates whether\
                    an interrupt should be raised when this rule matches
        :type enable_interrupt_on_counter_event: bool
        :param match_emergency_routing_status_to_incoming_packet: Indicates\
                    whether the emergency routing statuses should be matched\
                    against packets arriving at this router (if True), or if\
                    they should be matched against packets leaving this router\
                    (if False)
        :type match_emergency_routing_status_to_incoming_packet: bool
        :param destinations: Increment the counter if one or more of the given\
                    destinations match
        :type destinations: iterable of\
                    :py:class:`spinnman.model.diagnostic_filter_destination.DiagnosticFilterDestination`

        :param sources: Increment the counter if one or more of the given\
                    sources match (or None or empty list to match all)
        :type sources: iterable of\
                    :py:class:`spinnman.model.diagnostic_filter_source.DiagnosticFilterSource`
        :param payload_statuses: Increment the counter if one or more of the\
                    given payload statuses match  (or None or empty list to \
                    match all)
        :type payload_statuses: iterable of\
                    :py:class:`spinnman.model.diagnostic_filter_payload_status.DiagnosticFilterPayloadStatus`
        :param default_routing_statuses: Increment the counter if one or more\
                    of the given default routing statuses match  (or None or \
                    empty list to match all)
        :type default_routing_statuses: iterable of\
                    :py:class:`spinnman.model.diagnostic_filter_default_routing_status.DiagnosticFilterDefaultRoutingStatus`
        :param emergency_routing_statuses: Increment the counter if one or\
                    more of the given emergency routing statuses match  (or \
                    None or empty list to match all)
        :type emergency_routing_statuses: iterable of\
                    :py:class:`spinnman.model.diagnostic_filter_emergency_routing_status.DiagnosticFilterEmergencyRoutingStatus`
        :param packet_types: Increment the counter if one or more\
                    of the given packet types match  (or None or empty list to\
                    match all)
        :type packet_types: iterable of\
                    :py:class:`spinnman.model.diagnostic_filter_packet_type.DiagnosticFilterPacketType`
        """

        self._enable_interrupt_on_counter_event = \
            enable_interrupt_on_counter_event
        self._match_emergency_routing_status_to_incoming_packet = \
            match_emergency_routing_status_to_incoming_packet
        self._destinations = destinations
        self._sources = sources
        self._payload_statuses = payload_statuses
        self._default_routing_statuses = default_routing_statuses
        self._emergency_routing_statuses = emergency_routing_statuses
        self._packet_types = packet_types

    @property
    def enable_interrupt_on_counter_event(self):
        return self._enable_interrupt_on_counter_event

    @property
    def match_emergency_routing_status_to_incoming_packet(self):
        return self._match_emergency_routing_status_to_incoming_packet

    @property
    def destinations(self):
        return self._destinations

    @property
    def sources(self):
        return self._sources

    @property
    def payload_statuses(self):
        return self._payload_statuses

    @property
    def default_routing_statuses(self):
        return self._default_routing_statuses

    @property
    def emergency_routing_statuses(self):
        return self._emergency_routing_statuses

    @property
    def packet_types(self):
        return self._packet_types

    @property
    def filter_word(self):
        """ A word of data that can be written to the router to set up\
            the filter
        """
        data = 0
        if self._enable_interrupt_on_counter_event:
            data |= 1 << _ENABLE_INTERRUPT_OFFSET
        if not self._match_emergency_routing_status_to_incoming_packet:
            data |= 1 << _EMERGENCY_ROUTE_MODE_OFFSET
        data = _set_flags_in_word(data, self._destinations,
                                  DiagnosticFilterDestination,
                                  _DESTINATION_OFFSET)
        data = _set_flags_in_word(data, self._sources,
                                  DiagnosticFilterSource,
                                  _SOURCE_OFFSET)
        data = _set_flags_in_word(data, self._payload_statuses,
                                  DiagnosticFilterPayloadStatus,
                                  _PAYLOAD_OFFSET)
        data = _set_flags_in_word(data, self._default_routing_statuses,
                                  DiagnosticFilterDefaultRoutingStatus,
                                  _DEFAULT_ROUTE_OFFSET)
        data = _set_flags_in_word(data, self._emergency_routing_statuses,
                                  DiagnosticFilterEmergencyRoutingStatus,
                                  _EMERGENCY_ROUTE_OFFSET)
        data = _set_flags_in_word(data, self._packet_types,
                                  DiagnosticFilterPacketType,
                                  _PACKET_TYPE_OFFSET)

        return data

    @staticmethod
    def read_from_int(int_value):
        enable_interrupt_on_counter_event = (
            (int_value >> _ENABLE_INTERRUPT_OFFSET) & 0x1) == 1
        match_emergency_routing_status_to_incoming_packet = (
            (int_value >> _EMERGENCY_ROUTE_MODE_OFFSET) & 0x1) == 0
        destinations = _read_flags_from_word(
            int_value, DiagnosticFilterDestination, _DESTINATION_OFFSET)
        sources = _read_flags_from_word(
            int_value, DiagnosticFilterSource, _SOURCE_OFFSET)
        payload_statuses = _read_flags_from_word(
            int_value, DiagnosticFilterPayloadStatus, _PAYLOAD_OFFSET)
        default_routing_statuses = _read_flags_from_word(
            int_value, DiagnosticFilterDefaultRoutingStatus,
            _DEFAULT_ROUTE_OFFSET)
        emergency_routing_statuses = _read_flags_from_word(
            int_value, DiagnosticFilterEmergencyRoutingStatus,
            _EMERGENCY_ROUTE_OFFSET)
        packet_types = _read_flags_from_word(
            int_value, DiagnosticFilterPacketType, _PACKET_TYPE_OFFSET)

        return DiagnosticFilter(
            enable_interrupt_on_counter_event,
            match_emergency_routing_status_to_incoming_packet,
            destinations, sources, payload_statuses, default_routing_statuses,
            emergency_routing_statuses, packet_types)



from spinnman.model.cpu_info import CPUInfo
from spinnman import constants
from spinnman.messages.spinnaker_boot._system_variables.\
    _system_variable_boot_values import SystemVariableDefinition
from spinnman.utilities import utility_functions
from spinnman.messages.scp.impl.scp_read_memory_request \
    import SCPReadMemoryRequest
from spinnman.processes.abstract_multi_connection_process \
    import AbstractMultiConnectionProcess

import functools
import struct


VCPU = SystemVariableDefinition.cpu_information_base_address


class GetCPUInfoProcess(AbstractMultiConnectionProcess):

    def __init__(self, connection_selector):
        AbstractMultiConnectionProcess.__init__(self, connection_selector)
        self._vcpu_offsets = None
        self._cpu_info = list()

    def handle_read_vcpu_offset(self, x, y, response):
        offset = struct.unpack_from(
            VCPU.data_type.struct_code, response.data, response.offset)[0]
        self._vcpu_offsets[x, y] = offset

    def handle_response(self, x, y, p, response):
        self._cpu_info.append(CPUInfo(x, y, p, response.data, response.offset))

    def get_cpu_info(self, core_subsets):

        for core_subset in core_subsets:
            x = core_subset.x
            y = core_subset.y

            for p in core_subset.processor_ids:
                base_address = utility_functions.get_vcpu_address(p)
                self._send_request(SCPReadMemoryRequest(
                    x, y, base_address, constants.CPU_INFO_BYTES),
                    functools.partial(self.handle_response, x, y, p))
        self._finish()
        self.check_for_error()

        return self._cpu_info



from spinnman.messages.eieio.data_messages.eieio_data_message\
    import EIEIODataMessage
from spinnman.exceptions import SpinnmanInvalidParameterException
from spinnman.messages.eieio.data_messages.eieio_key_payload_data_element \
    import EIEIOKeyPayloadDataElement


class EIEIOWithPayloadDataMessage(EIEIODataMessage):
    """ An EIEIO message with a payload
    """

    def __init__(self, eieio_header, data=None, offset=0):
        EIEIODataMessage.__init__(self, eieio_header, data, offset)

        if eieio_header.eieio_type.payload_bytes == 0:
            raise SpinnmanInvalidParameterException(
                "eieio_header", eieio_header,
                "This message should have a payload, but the header indicates"
                " that it doesn't")

    def add_key_and_payload(self, key, payload):
        """ Adds a key and payload to the packet

        :param key: The key to add
        :type key: int
        :param payload: The payload to add
        :type payload: int
        :raise SpinnmanInvalidParameterException: If the key or payload is too\
                    big for the format
        """
        if key > self._eieio_header.eieio_type.max_value:
            raise SpinnmanInvalidParameterException(
                "key", key,
                "Larger than the maximum allowed of {}".format(
                    self._eieio_header.eieio_type.max_value))
        if payload > self._eieio_header.eieio_type.max_value:
            raise SpinnmanInvalidParameterException(
                "payload", payload,
                "Larger than the maximum allowed of {}".format(
                    self._eieio_header.eieio_type.max_value))
        EIEIODataMessage.add_element(
            self, EIEIOKeyPayloadDataElement(key, payload,
                                             self._eieio_header.is_time))



from spinnman.messages.eieio.eieio_type import EIEIOType
from spinnman.messages.eieio.data_messages.eieio_with_payload_data_message\
    import EIEIOWithPayloadDataMessage
from spinnman.messages.eieio.data_messages.eieio_data_header\
    import EIEIODataHeader
from spinnman.messages.eieio.data_messages.eieio_data_message\
    import EIEIODataMessage


class EIEIO16BitWithPayloadLowerKeyPrefixDataMessage(
        EIEIOWithPayloadDataMessage):
    """ An EIEIO packet containing 16 bit events and payloads with a key\
        prefix to be applied to the lower end of the key
    """
    def __init__(self, key_prefix, count=0, data=None, offset=0):
        EIEIOWithPayloadDataMessage.__init__(
            self, EIEIODataHeader(EIEIOType.KEY_PAYLOAD_16_BIT,
                                  prefix=key_prefix, count=count),
            data, offset)

    @staticmethod
    def get_min_packet_length():
        return EIEIODataMessage.min_packet_length(
            EIEIOType.KEY_PAYLOAD_16_BIT, is_prefix=True)



from spinnman.messages.scp.abstract_messages.abstract_scp_request\
    import AbstractSCPRequest
from spinnman.messages.sdp.sdp_header import SDPHeader
from spinnman.messages.sdp.sdp_flag import SDPFlag
from spinnman.messages.scp.scp_request_header import SCPRequestHeader
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.impl.scp_check_ok_response import SCPCheckOKResponse


class SCPLEDRequest(AbstractSCPRequest):
    """ A request to change the state of an LED
    """

    def __init__(self, x, y, cpu, led_states):
        """

        :param x: The x-coordinate of the chip, between 0 and 255
        :type x: int
        :param y: The y-coordinate of the chip, between 0 and 255
        :type y: int
        :param cpu: The CPU-number to use to set the LED.
        :type cpu: int
        :param led_states: A dictionary mapping LED index to state with 0 being
                           off, 1 on and 2 inverted.
        :type led_states: dict
        """
        encoded_led_states = 0
        for led, state in led_states.items():
            encoded_led_states |= {0: 2, 1: 3, 2: 1}[state] << (2 * led)

        super(SCPLEDRequest, self).__init__(
            SDPHeader(
                flags=SDPFlag.REPLY_EXPECTED, destination_port=0,
                destination_cpu=cpu, destination_chip_x=x,
                destination_chip_y=y),
            SCPRequestHeader(command=SCPCommand.CMD_LED),
            argument_1=encoded_led_states)

    def get_scp_response(self):
        """ See\
            :py:meth:`spinnman.messages.scp.abstract_scp_request.AbstractSCPRequest.get_scp_response`
        """
        return SCPCheckOKResponse("Set LED", "CMD_LED")



from spinnman.messages.scp.impl.scp_iptag_info_response \
    import SCPIPTagInfoResponse
from spinnman.messages.scp.abstract_messages.abstract_scp_request \
    import AbstractSCPRequest
from spinnman.messages.sdp.sdp_flag import SDPFlag
from spinnman.messages.sdp.sdp_header import SDPHeader
from spinnman.messages.scp.scp_request_header import SCPRequestHeader
from spinnman.messages.scp.enums.scp_command import SCPCommand

_IPTAG_TTO = (4 << 16)


class SCPIPTagTTORequest(AbstractSCPRequest):
    """ An SCP request to set the transient timeout for future SCP requests
    """

    def __init__(self, x, y, tag_timeout):
        """

        :param x: The x-coordinate of the chip to run on, between 0 and 255
        :type x: int
        :param y: The y-coordinate of the chip to run on, between 0 and 255
        :type y: int
        :param tag_timeout: The timeout value, via the
            IPTAG_TIME_OUT_WAIT_TIMES enum located in spinnman.constants
        """

        super(SCPIPTagTTORequest, self).__init__(
            SDPHeader(flags=SDPFlag.REPLY_EXPECTED, destination_port=0,
                      destination_cpu=0, destination_chip_x=x,
                      destination_chip_y=y),
            SCPRequestHeader(command=SCPCommand.CMD_IPTAG),
            argument_1=_IPTAG_TTO, argument_2=tag_timeout)

    def get_scp_response(self):
        """ See\
            :py:meth:`spinnman.messages.scp.abstract_scp_request.AbstractSCPRequest.get_scp_response`
        """
        return SCPIPTagInfoResponse()



from spinnman.messages.scp.abstract_messages.abstract_scp_request\
    import AbstractSCPRequest
from spinnman.messages.sdp.sdp_header import SDPHeader
from spinnman.messages.sdp.sdp_flag import SDPFlag
from spinnman.messages.scp.scp_request_header import SCPRequestHeader
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.impl.scp_check_ok_response import SCPCheckOKResponse

_NNP_FORWARD_RETRY = (0x3f << 24) | (0x18 << 16)
_NNP_FLOOD_FILL_START = 6


class SCPFloodFillDataRequest(AbstractSCPRequest):
    """ A request to start a flood fill of data
    """

    def __init__(self, nearest_neighbour_id, block_no, base_address, data,
                 offset=0, length=None):
        """

        :param nearest_neighbour_id: The id of the packet, between 0 and 127
        :type nearest_neighbour_id: int
        :param block_no: Which block this block is, between 0 and 255
        :type block_no: int
        :param base_address: The base address where the data is to be loaded
        :type base_address: int
        :param data: The data to load, between 4 and 256 bytes and the size\
                    must be divisible by 4
        :type data: bytearray
        """
        self._size = length
        self._offset = offset
        self._data_to_write = data
        if length is None:
            self._size = len(data)

        argument_1 = _NNP_FORWARD_RETRY | nearest_neighbour_id
        argument_2 = (block_no << 16) | (((self._size / 4) - 1) << 8)

        super(SCPFloodFillDataRequest, self).__init__(
            SDPHeader(
                flags=SDPFlag.REPLY_EXPECTED, destination_port=0,
                destination_cpu=0,
                destination_chip_x=self.DEFAULT_DEST_X_COORD,
                destination_chip_y=self.DEFAULT_DEST_Y_COORD),
            SCPRequestHeader(command=SCPCommand.CMD_FFD),
            argument_1=argument_1, argument_2=argument_2,
            argument_3=base_address, data=None)

    @property
    def bytestring(self):
        datastring = super(SCPFloodFillDataRequest, self).bytestring
        data = self._data_to_write[self._offset:self._offset + self._size]
        return datastring + bytes(data)

    def get_scp_response(self):
        return SCPCheckOKResponse("Flood Fill", "CMD_NNP:NNP_FFS")



from spinnman.messages.scp.abstract_messages.abstract_scp_request\
    import AbstractSCPRequest
from spinnman.messages.sdp.sdp_header import SDPHeader
from spinnman.messages.sdp.sdp_flag import SDPFlag
from spinnman.messages.scp.scp_request_header import SCPRequestHeader
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.impl.scp_check_ok_response import SCPCheckOKResponse

_IPTAG_CLEAR = 3


class SCPIPTagClearRequest(AbstractSCPRequest):
    """ An SCP Request to clear an IP Tag
    """

    def __init__(self, x, y, tag):
        """
        :param x: The x-coordinate of a chip, between 0 and 255
        :type x: int
        :param y: The y-coordinate of a chip, between 0 and 255
        :type y: int
        :param tag: The tag, between 0 and 7
        :type tag: int
        """
        super(SCPIPTagClearRequest, self).__init__(
            SDPHeader(
                flags=SDPFlag.REPLY_EXPECTED, destination_port=0,
                destination_cpu=0, destination_chip_x=x,
                destination_chip_y=y),
            SCPRequestHeader(command=SCPCommand.CMD_IPTAG),
            argument_1=(_IPTAG_CLEAR << 16) | tag)

    def get_scp_response(self):
        return SCPCheckOKResponse("Clear IP Tag", "CMD_IPTAG")



from spinnman.processes.abstract_multi_connection_process_connection_selector \
    import AbstractMultiConnectionProcessConnectionSelector


class MultiConnectionProcessMostDirectConnectionSelector(
        AbstractMultiConnectionProcessConnectionSelector):
    """
    MultiConnectionProcessMostDirectConnectionSelector: selector that goes
    for the most direct connection for the message
    """

    def __init__(self, machine, connections):
        AbstractMultiConnectionProcessConnectionSelector.__init__(
            self, connections)
        self._machine = machine
        self._connections = dict()
        self._first_connection = None
        for connection in connections:
            if connection.chip_x == 0 and connection.chip_y == 0:
                self._first_connection = connection
            self._connections[
                (connection.chip_x, connection.chip_y)] = connection
        if self._first_connection is None:
            self._first_connection = next(iter(connections))

    def set_machine(self, new_machine):
        self._machine = new_machine

    def get_next_connection(self, message):
        if self._machine is None or len(self._connections) == 1:
            return self._first_connection

        chip = self._machine.get_chip_at(
            message.sdp_header.destination_chip_x,
            message.sdp_header.destination_chip_y)
        key = (chip.nearest_ethernet_x, chip.nearest_ethernet_y)

        if key not in self._connections:
            return self._first_connection
        return self._connections[key]



import struct

from spinnman.model.enums.p2p_table_route import P2PTableRoute


class P2PTable(object):
    """ Represents a P2P table read from the machine
    """

    def __init__(self, width, height, column_data):
        self._routes = dict()
        self._width = width
        self._height = height
        for x in range(len(column_data)):
            y = 0
            pos = 0
            while y < height:
                (data, offset) = column_data[x]
                next_word = struct.unpack_from(
                    "<I", data, offset + (pos * 4))[0]
                pos += 1
                for entry in range(min(8, height - y)):
                    route = P2PTableRoute((next_word >> (3 * entry)) & 0b111)
                    if route is not P2PTableRoute.NONE:
                        self._routes[x, y] = route
                    y += 1

    @staticmethod
    def get_n_column_bytes(height):
        """ Get the number of bytes to be read for each column of the table

        :param height: The height of the machine
        """
        return ((height + 7) // 8) * 4

    @staticmethod
    def get_column_offset(column):
        """ Get the offset of the next column in the table from the P2P base\
            address

        :param column: The column to be read
        """
        return (((256 * column) // 8) * 4)

    @property
    def width(self):
        """ The width of the machine that this table represents
        """
        return self._width

    @property
    def height(self):
        """ The height of the machine that this table represents
        """
        return self._height

    def iterchips(self):
        """ Get an iterator of tuples of (x, y) coordinates in the table
        """
        return self._routes.iterkeys()

    def is_route(self, x, y):
        """ Determines if there is a route in the P2P table to the given chip

        :param x: The x-coordinate of the chip to look up
        :param y: The y-coordinate of the chip to look up
        """
        return (
            (x, y) in self._routes and
            self._routes[(x, y)] != P2PTableRoute.NONE)

    def get_route(self, x, y):
        """ Get the route to follow from this chip to the given chip

        :param x: The x-coordinate of the chip to find the route to
        :param y: The y-coordinate of the chip to find the route to
        """
        if (x, y) in self._routes:
            return self._routes[x, y]
        return P2PTableRoute.NONE



from spinnman.messages.scp.abstract_messages.abstract_scp_request \
    import AbstractSCPRequest
from spinnman.messages.sdp.sdp_flag import SDPFlag
from spinnman.messages.sdp.sdp_header import SDPHeader
from spinnman.messages.scp.scp_request_header import SCPRequestHeader
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.impl.scp_check_ok_response import SCPCheckOKResponse

_WAIT_FLAG = 0x1 << 18


class SCPApplicationRunRequest(AbstractSCPRequest):
    """ An SCP request to run an application loaded on a chip
    """

    def __init__(self, app_id, x, y, processors, wait=False):
        """

        :param app_id: The id of the application to run, between 16 and 255
        :type app_id: int
        :param x: The x-coordinate of the chip to run on, between 0 and 255
        :type x: int
        :param y: The y-coordinate of the chip to run on, between 0 and 255
        :type y: int
        :param processors: The processors on the chip where the executable\
                    should be started, between 1 and 17
        :type processors: list of int
        :param wait: True if the processors should enter a "wait" state on\
                    starting
        :type wait: bool
        """
        processor_mask = 0
        if processors is not None:
            for processor in processors:
                processor_mask |= (1 << processor)

        processor_mask |= (app_id << 24)
        if wait:
            processor_mask |= _WAIT_FLAG

        super(SCPApplicationRunRequest, self).__init__(
            SDPHeader(flags=SDPFlag.REPLY_EXPECTED, destination_port=0,
                      destination_cpu=0, destination_chip_x=x,
                      destination_chip_y=y),
            SCPRequestHeader(command=SCPCommand.CMD_AR),
            argument_1=processor_mask)

    def get_scp_response(self):
        """ See\
            :py:meth:`spinnman.messages.scp.abstract_scp_request.AbstractSCPRequest.get_scp_response`
        """
        return SCPCheckOKResponse("Run Application", "CMD_AR")



from spinnman import constants
from spinnman.messages.eieio.command_messages.eieio_command_message\
    import EIEIOCommandMessage
from spinnman.messages.eieio.command_messages.eieio_command_header\
    import EIEIOCommandHeader


class EventStopRequest(EIEIOCommandMessage):
    """ Packet used for the buffering input technique which causes the parser\
        of the input packet to terminate its execution
    """
    def __init__(self):
        EIEIOCommandMessage.__init__(
            self, EIEIOCommandHeader(
                constants.EIEIO_COMMAND_IDS.EVENT_STOP.value))



from spinnman.messages.eieio.data_messages.eieio_data_message\
    import EIEIODataMessage
from spinnman.exceptions import SpinnmanInvalidParameterException
from spinnman.messages.eieio.data_messages.eieio_key_data_element\
    import EIEIOKeyDataElement


class EIEIOWithoutPayloadDataMessage(EIEIODataMessage):
    """ An EIEIO message without a payload
    """

    def __init__(self, eieio_header, data=None, offset=0):
        EIEIODataMessage.__init__(self, eieio_header, data, offset)

        if eieio_header.eieio_type.payload_bytes != 0:
            raise SpinnmanInvalidParameterException(
                "eieio_header", eieio_header,
                "This message should have no payload, but the header indicates"
                " that it does")

    def add_key(self, key):
        """ Add a key to the packet

        :param key: The key to add
        :type key: int
        :raise SpinnmanInvalidParameterException: If the key is too\
                    big for the format
        """
        if key > self._eieio_header.eieio_type.max_value:
            raise SpinnmanInvalidParameterException(
                "key", key,
                "Larger than the maximum allowed of {}".format(
                    self._eieio_header.eieio_type.max_value))
        EIEIODataMessage.add_element(self, EIEIOKeyDataElement(key))



"""
SCPPowerRequest
"""

# spinnman imports
from spinnman.messages.scp.abstract_messages.abstract_scp_bmp_request import \
    AbstractSCPBMPRequest
from spinnman.messages.scp.impl.scp_check_ok_response import SCPCheckOKResponse
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.scp_request_header import SCPRequestHeader


class SCPPowerRequest(AbstractSCPBMPRequest):
    """ An SCP request for the BMP to power on or power off a rack of boards
    """

    def __init__(self, power_command, boards, delay=0.0):
        """
        :param power_command: The power command being sent
        :type power_command:\
                :py:class:`spinnman.messages.scp.scp_power_command.SCPPowerCommand`
        :param boards: The boards on the same backplane to power on or off
        :type boards: int or iterable of int
        :param delay: Number of seconds delay between power state changes of\
                the different boards.
        :type delay: int
        :rtype: None
        """

        arg1 = (int(delay * 1000) << 16) | power_command.value
        arg2 = self.get_board_mask(boards)

        AbstractSCPBMPRequest.__init__(
            self, boards,
            SCPRequestHeader(command=SCPCommand.CMD_BMP_POWER),
            argument_1=arg1, argument_2=arg2)

    def get_scp_response(self):
        """ Get the response from the powering message
        """
        return SCPCheckOKResponse("powering request", "CMD_BMP_POWER")



from spinnman.messages.scp.abstract_messages.abstract_scp_response \
    import AbstractSCPResponse
from spinnman.messages.scp.enums.scp_result import SCPResult
from spinnman.exceptions import SpinnmanUnexpectedResponseCodeException


class SCPReadMemoryResponse(AbstractSCPResponse):
    """ An SCP response to a request to read a region of memory on a chip
    """

    def __init__(self):
        """
        """
        super(SCPReadMemoryResponse, self).__init__()
        self._data = None
        self._offset = None

    def read_data_bytestring(self, data, offset):
        if self._scp_response_header.result != SCPResult.RC_OK:
            raise SpinnmanUnexpectedResponseCodeException(
                "Read", "CMD_READ", self._scp_response_header.result)
        self._data = data
        self._offset = offset
        self._length = len(data) - offset

    @property
    def data(self):
        """ The data read - note that the data starts at offset

        :rtype: bytearray
        """
        return self._data

    @property
    def offset(self):
        """ The offset where the valid data starts

        :rtype: int
        """
        return self._offset

    @property
    def length(self):
        """ The length of the valid data

        :rtype: int
        """
        return self._length



from spinnman import constants
import struct

from spinnman.messages.scp.enums.scp_result import SCPResult
from spinnman.connections.udp_packet_connections import udp_utils
from spinnman.connections.udp_packet_connections.udp_sdp_connection \
    import UDPSDPConnection
from spinnman.connections.abstract_classes.abstract_scp_sender \
    import AbstractSCPSender
from spinnman.connections.abstract_classes.abstract_scp_receiver \
    import AbstractSCPReceiver


class UDPSCAMPConnection(UDPSDPConnection, AbstractSCPSender,
                         AbstractSCPReceiver):
    """ A UDP connection to SCAMP on the board
    """

    def __init__(
            self, chip_x=255, chip_y=255, local_host=None, local_port=None,
            remote_host=None, remote_port=None):
        """

        :param chip_x: The x-coordinate of the chip on the board with this\
                remote_host
        :type chip_x: int
        :param chip_y: The y-coordinate of the chip on the board with this\
                remote_host
        :type chip_y: int
        :param local_host: The optional ip address or host name of the local\
                interface to listen on
        :type local_host: str
        :param local_port: The optional local port to listen on
        :type local_port: int
        :param remote_host: The optional remote host name or ip address to\
                send messages to.  If not specified, sending will not be\
                possible using this connection
        :type remote_host: str
        :param remote_port: The optional remote port number to send messages\
                to.  If not specified, sending will not be possible using this\
                connection
        :type remote_port: int
        """
        if remote_port is None:
            remote_port = constants.SCP_SCAMP_PORT
        UDPSDPConnection.__init__(
            self, chip_x, chip_y, local_host, local_port, remote_host,
            remote_port)
        AbstractSCPReceiver.__init__(self)
        AbstractSCPSender.__init__(self)

    @property
    def chip_x(self):
        return self._chip_x

    @property
    def chip_y(self):
        return self._chip_y

    def update_chip_coordinates(self, x, y):
        self._chip_x = x
        self._chip_y = y

    def get_scp_data(self, scp_request):
        udp_utils.update_sdp_header_for_udp_send(scp_request.sdp_header,
                                                 self._chip_x, self._chip_y)
        return struct.pack("<2x") + scp_request.bytestring

    def receive_scp_response(self, timeout=1.0):
        data = self.receive(timeout)
        result, sequence = struct.unpack_from("<2H", data, 10)
        return SCPResult(result), sequence, data, 2

    def send_scp_request(self, scp_request):
        self.send(self.get_scp_data(scp_request))

    def __repr__(self):
        return \
            "UDPSCAMPConnection(chip_x={}, chip_y={}, local_host={}," \
            " local_port={}, remote_host={}, remote_port={})".format(
                self._chip_x, self._chip_y, self.local_ip_address,
                self.local_port, self.remote_ip_address, self.remote_port)



from spinnman.messages.scp.impl.scp_flood_fill_end_request \
    import SCPFloodFillEndRequest
from spinnman.messages.scp.impl.scp_flood_fill_start_request \
    import SCPFloodFillStartRequest
from spinnman.messages.scp.impl.scp_flood_fill_data_request \
    import SCPFloodFillDataRequest
from spinnman.processes.abstract_multi_connection_process \
    import AbstractMultiConnectionProcess
from spinnman import constants

import math


class WriteMemoryFloodProcess(AbstractMultiConnectionProcess):
    """ A process for writing memory
    """

    def __init__(self, connection_selector):
        AbstractMultiConnectionProcess.__init__(self, connection_selector)

    def _start_flood_fill(self, n_bytes, nearest_neighbour_id):
        n_blocks = int(math.ceil(math.ceil(n_bytes / 4.0) /
                                 constants.UDP_MESSAGE_MAX_SIZE))
        self._send_request(
            SCPFloodFillStartRequest(nearest_neighbour_id, n_blocks))
        self._finish()
        self.check_for_error()

    def _end_flood_fill(self, nearest_neighbour_id):
        self._send_request(SCPFloodFillEndRequest(nearest_neighbour_id))
        self._finish()
        self.check_for_error()

    def write_memory_from_bytearray(self, nearest_neighbour_id, base_address,
                                    data, offset, n_bytes):
        self._start_flood_fill(n_bytes, nearest_neighbour_id)

        data_offset = offset
        offset = base_address
        block_no = 0
        while n_bytes > 0:

            bytes_to_send = int(n_bytes)
            if bytes_to_send > constants.UDP_MESSAGE_MAX_SIZE:
                bytes_to_send = constants.UDP_MESSAGE_MAX_SIZE

            self._send_request(
                SCPFloodFillDataRequest(nearest_neighbour_id, block_no, offset,
                                        data, data_offset, bytes_to_send))

            block_no += 1
            n_bytes -= bytes_to_send
            offset += bytes_to_send
            data_offset += bytes_to_send
        self._finish()
        self.check_for_error()

        self._end_flood_fill(nearest_neighbour_id)

    def write_memory_from_reader(self, nearest_neighbour_id, base_address,
                                 data, n_bytes):
        self._start_flood_fill(n_bytes, nearest_neighbour_id)

        offset = base_address
        block_no = 0
        while n_bytes > 0:

            bytes_to_send = int(n_bytes)
            if bytes_to_send > constants.UDP_MESSAGE_MAX_SIZE:
                bytes_to_send = constants.UDP_MESSAGE_MAX_SIZE
            data_array = data.read(bytes_to_send)

            self._send_request(
                SCPFloodFillDataRequest(nearest_neighbour_id, block_no, offset,
                                        data_array, 0, len(data_array)))

            block_no += 1
            n_bytes -= bytes_to_send
            offset += bytes_to_send
        self._finish()
        self.check_for_error()

        self._end_flood_fill(nearest_neighbour_id)



from enum import Enum


class DiagnosticFilterDestination(Enum):
    """ Destination flags for the diagnostic filters.
        Note that only one has to match for the counter to be incremented
    """
    DUMP = (0, "Destination is to dump the packet")
    LOCAL = (1, "Destination is a local core (but not the monitor core)")
    LOCAL_MONITOR = (2, "Destination is the local monitor core")
    LINK_0 = (3, "Destination is link 0")
    LINK_1 = (4, "Destination is link 1")
    LINK_2 = (5, "Destination is link 2")
    LINK_3 = (6, "Destination is link 3")
    LINK_4 = (7, "Destination is link 4")
    LINK_5 = (8, "Destination is link 5")

    def __new__(cls, value, doc=""):
        obj = object.__new__(cls)
        obj._value_ = value
        return obj

    def __init__(self, value, doc=""):
        self._value_ = value
        self.__doc__ = doc



from enum import Enum


class DiagnosticFilterDefaultRoutingStatus(Enum):
    """ Default routing flags for the diagnostic filters.
        Note that only one has to match for the counter to be incremented
    """
    DEFAULT_ROUTED = (0, "Packet is to be default routed")
    NON_DEFAULT_ROUTED = (1, "Packet is not to be default routed")

    def __new__(cls, value, doc=""):
        obj = object.__new__(cls)
        obj._value_ = value
        return obj

    def __init__(self, value, doc=""):
        self._value_ = value
        self.__doc__ = doc



from spinnman.messages.scp.abstract_messages.abstract_scp_request\
    import AbstractSCPRequest
from spinnman.messages.sdp.sdp_header import SDPHeader
from spinnman.messages.sdp.sdp_flag import SDPFlag
from spinnman.messages.scp.scp_request_header import SCPRequestHeader
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.impl.scp_router_alloc_response\
    import SCPRouterAllocResponse
from spinnman.messages.scp.enums.scp_alloc_free_type import SCPAllocFreeType


class SCPRouterAllocRequest(AbstractSCPRequest):
    """ An SCP Request to allocate space for routing entries
    """

    def __init__(self, x, y, app_id, n_entries):
        """

        :param x: The x-coordinate of the chip to allocate on, between 0 and\
                    255
        :type x: int
        :param y: The y-coordinate of the chip to allocate on, between 0 and\
                    255
        :type y: int
        :param app_id: The id of the application, between 0 and 255
        :type app_id: int
        :param n_entries: The number of entries to allocate
        :type n_entries: int
        """
        super(SCPRouterAllocRequest, self).__init__(
            SDPHeader(
                flags=SDPFlag.REPLY_EXPECTED, destination_port=0,
                destination_cpu=0, destination_chip_x=x,
                destination_chip_y=y),
            SCPRequestHeader(command=SCPCommand.CMD_ALLOC),
            argument_1=(
                (app_id << 8) |
                SCPAllocFreeType.ALLOC_ROUTING.value),  # @UndefinedVariable
            argument_2=n_entries)

    def get_scp_response(self):
        return SCPRouterAllocResponse()



from spinnman import constants
from spinnman.messages.eieio.command_messages.eieio_command_message\
    import EIEIOCommandMessage
from spinnman.messages.eieio.command_messages.eieio_command_header\
    import EIEIOCommandHeader


class PaddingRequest(EIEIOCommandMessage):
    """ Packet used to pad space in the buffering area, if needed
    """
    def __init__(self):
        EIEIOCommandMessage.__init__(
            self, EIEIOCommandHeader(
                constants.EIEIO_COMMAND_IDS.EVENT_PADDING.value))

    @staticmethod
    def get_min_packet_length():
        return 2



from spinnman.messages.eieio.eieio_type import EIEIOType
from spinnman.messages.eieio.eieio_prefix import EIEIOPrefix
from spinnman.messages.eieio.data_messages.eieio_without_payload_data_message\
    import EIEIOWithoutPayloadDataMessage
from spinnman.messages.eieio.data_messages.eieio_data_header\
    import EIEIODataHeader
from spinnman.messages.eieio.data_messages.eieio_data_message\
    import EIEIODataMessage


class EIEIO32BitTimedPayloadPrefixUpperKeyPrefixDataMessage(
        EIEIOWithoutPayloadDataMessage):
    """ An EIEIO packet containing 32 bit events, a key prefix to be applied\
        to the upper end of the key, and a timestamp for the events
    """
    def __init__(self, key_prefix, timestamp, count=0, data=None, offset=0):
        EIEIOWithoutPayloadDataMessage.__init__(
            self, EIEIODataHeader(EIEIOType.KEY_32_BIT, payload_base=timestamp,
                                  is_time=True, prefix=key_prefix,
                                  prefix_type=EIEIOPrefix.UPPER_HALF_WORD,
                                  count=count),
            data, offset)

    @staticmethod
    def get_min_packet_length():
        return EIEIODataMessage.min_packet_length(
            EIEIOType.KEY_32_BIT, is_payload_base=True, is_prefix=True)



from spinnman.exceptions import SpinnmanInvalidParameterException
from spinnman.messages.eieio.eieio_type import EIEIOType
from spinnman.messages.eieio.data_messages.abstract_eieio_data_element \
    import AbstractEIEIODataElement

import struct


class EIEIOKeyPayloadDataElement(AbstractEIEIODataElement):
    """ A data element that contains a key and a payload
    """

    def __init__(self, key, payload, payload_is_timestamp=False):
        self._key = key
        self._payload = payload
        self._payload_is_timestamp = payload_is_timestamp

    @property
    def key(self):
        return self._key

    @property
    def payload(self):
        return self._payload

    @property
    def payload_is_timestamp(self):
        return self._payload_is_timestamp

    def get_bytestring(self, eieio_type):
        if eieio_type.payload_bytes == 0:
            raise SpinnmanInvalidParameterException(
                "eieio_type", eieio_type,
                "The type specifies no payload, but this element has a"
                " payload")
        if eieio_type == EIEIOType.KEY_PAYLOAD_16_BIT:
            return struct.pack("<HH", self._key, self._payload)
        elif eieio_type == EIEIOType.KEY_PAYLOAD_32_BIT:
            return struct.pack("<II", self._key, self._payload)
        else:
            raise SpinnmanInvalidParameterException(
                "eieio_type", eieio_type, "Unknown type")

    def __str__(self):
        return "EIEIOKeyPayloadDataElement:{}:{}".format(hex(self._key),
                                                         hex(self._payload))

    def __repr__(self):
        return self.__str__()



"""
ScpReadAdcRequest
"""
from spinnman.messages.scp.abstract_messages.abstract_scp_bmp_request import \
    AbstractSCPBMPRequest
from spinnman.messages.scp.impl.scp_read_adc_response import SCPReadADCResponse
from spinnman.messages.scp.enums.scp_command import SCPCommand
from spinnman.messages.scp.scp_request_header import SCPRequestHeader
from spinnman.messages.scp.enums.scp_bmp_info_type import SCPBMPInfoType


class SCPReadADCRequest(AbstractSCPBMPRequest):
    """ SCP Request for the data from the BMP including voltages and\
        temperature.
    """

    def __init__(self, board):
        """

        :param board: which board to request the adc register from
        :rtype: None
        """
        AbstractSCPBMPRequest.__init__(
            self, board,
            SCPRequestHeader(command=SCPCommand.CMD_BMP_INFO),
            argument_1=SCPBMPInfoType.ADC)

    def get_scp_response(self):
        """
        """
        return SCPReadADCResponse()
